Type: Package
Package: cardelino
Title: Clone Identification from Single Cell Data
Version: 1.14.0
Authors@R: 
    c(person(given = "Jeffrey", 
             family = "Pullin",
             role = "aut",
             email = "jpullin@svi.edu.au"),
      person(given = "Yuanhua",
             family = "Huang",
             role = "aut",
             email = "yuanhua@hku.hk"),
      person(given = "Davis",
             family = "McCarthy",
             role = c("aut", "cre"),
             email = "dmccarthy@svi.edu.au"))
Description: Methods to infer clonal tree configuration for a
        population of cells using single-cell RNA-seq data (scRNA-seq),
        and possibly other data modalities. Methods are also provided
        to assign cells to inferred clones and explore differences in
        gene expression between clones. These methods can flexibly
        integrate information from imperfect clonal trees inferred
        based on bulk exome-seq data, and sparse variant alleles
        expressed in scRNA-seq data. A flexible beta-binomial error
        model that accounts for stochastic dropout events as well as
        systematic allelic imbalance is used.
License: GPL-3
URL: https://github.com/single-cell-genetics/cardelino
BugReports: https://github.com/single-cell-genetics/cardelino/issues
Depends: R (>= 4.2), stats
Imports: combinat, GenomeInfoDb, GenomicRanges, ggplot2, ggtree,
        Matrix, matrixStats, methods, pheatmap, snpStats, S4Vectors,
        utils, VariantAnnotation, vcfR
Suggests: BiocStyle, foreach, knitr, pcaMethods, rmarkdown, testthat,
        VGAM
Enhances: doMC
VignetteBuilder: knitr
biocViews: SingleCell, RNASeq, Visualization, Transcriptomics,
        GeneExpression, Sequencing, Software, ExomeSeq
Encoding: UTF-8
NeedsCompilation: yes
RoxygenNote: 7.2.1
Config/pak/sysreqs: libcairo2-dev cmake libfontconfig1-dev
        libfreetype6-dev make libbz2-dev libicu-dev liblzma-dev
        libpng-dev libuv1-dev libxml2-dev libssl-dev xz-utils
        zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:59:16 UTC
RemoteUrl: https://github.com/bioc/cardelino
RemoteRef: RELEASE_3_23
RemoteSha: 65677bd3d8e5cc98920a03b4da08452ef614e995
Packaged: 2026-04-29 18:39:08 UTC; root
Author: Jeffrey Pullin [aut],
  Yuanhua Huang [aut],
  Davis McCarthy [aut, cre]
Maintainer: Davis McCarthy <dmccarthy@svi.edu.au>
Built: R 4.6.0; ; 2026-04-29 20:24:18 UTC; unix

# Generated by roxygen2: do not edit by hand

export(assign_cells_to_clones)
export(assign_scores)
export(binaryPRC)
export(binaryROC)
export(clone_id)
export(clone_id_EM)
export(clone_id_Gibbs)
export(colMatch)
export(donor_read_simulator)
export(get_snp_matrices)
export(get_tree)
export(heat_matrix)
export(load_GT_vcf)
export(load_cellSNP_vcf)
export(mixBinom)
export(mtx_to_df)
export(multiPRC)
export(plot_config_diffs)
export(plot_tree)
export(predMixBinom)
export(prob_heatmap)
export(pub.theme)
export(read_vcf)
export(rowArgmax)
export(rowMax)
export(sample_seq_depth)
export(sample_tree_SNV)
export(sim_read_count)
export(vc_heatmap)
import(ggplot2)
import(matrixStats)
import(pheatmap)
import(snpStats)
import(stats)
import(utils)
importFrom(GenomeInfoDb,seqlengths)
importFrom(GenomeInfoDb,seqlevelsStyle)
importFrom(GenomicRanges,findOverlaps)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,subjectHits)
importFrom(VariantAnnotation,alt)
importFrom(VariantAnnotation,geno)
importFrom(VariantAnnotation,genotypeToSnpMatrix)
importFrom(VariantAnnotation,isSNV)
importFrom(VariantAnnotation,readVcf)
importFrom(VariantAnnotation,ref)
importFrom(combinat,permn)
importFrom(ggtree,geom_tiplab)
importFrom(ggtree,ggtree)
importFrom(methods,as)
importFrom(methods,is)

# cardelino: clone identification from single-cell data <img src="inst/cardelino_sticker.png" align="right" width="160" />

<!-- badges: start -->
[![R build status](https://github.com/single-cell-genetics/cardelino/workflows/R-CMD-check/badge.svg)](https://github.com/single-cell-genetics/cardelino/actions)
<!-- badges: end -->

**cardelino** contains a Bayesian method to infer clonal structure for a 
population of cells using single-cell RNA-seq data (and possibly other data 
modalities). 

In its main mode **cardelino** requires: 

* An imperfect clonal tree structure inferred using, for example [Canopy](https://cran.r-project.org/web/packages/Canopy/index.html), from bulk or single-cell DNA sequencing data (e.g. bulk whole exome sequencing data).

* Single-cell RNA sequencing data from which cell-specific somatic variants are called using, for example [cellsnp-lite](https://github.com/single-cell-genetics/cellsnp-lite).

## Installation

### Release version

You can install the release version of `cardelino` from BioConductor:

```r
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("cardelino")
```

### Development version

The development version of `cardelino` can be installed using the 
[`remotes`](https://remotes.r-lib.org/) package thus:

```r
# install.packages("remotes")
remotes::install_github("single-cell-genetics/cardelino")
```

## Getting started

The best place to start are the vignettes. From inside an R session, load 
`cardelino` and then browse the vignettes:

```r
library(cardelino)
browseVignettes("cardelino")
```

## Notes on donor deconvolution

The donor demultiplex function, named Vireo, was previously supported in this 
R package, but now has been re-implemented in Python, which is more memory 
efficient and easier to run via a command line. We, therefore, highly recommend 
you switch to the Python version: https://vireoSNP.readthedocs.io. 

The vireo function is not supported from version >=0.5.0. If you want to use the
R functions, please use the version ==0.4.2 or lower. You can also find it in a
separate branch in this repository: [with_vireo branch](https://github.com/single-cell-genetics/cardelino/tree/with_vireo)
or use the [donor_id.R](https://github.com/single-cell-genetics/cardelino/blob/with_vireo/R/donor_id.R) 
file directly. However, using the Python implementation of Vireo is **strongly** recommended. 

## Code of Conduct

Please note that the cardelino project is released with a [Contributor Code of Conduct](http://bioconductor.org/about/code-of-conduct/).
By contributing to this project, you agree to abide by its terms.

## Citation

If you find cardelino helpful please consider citing: 

> McCarthy, D.J., Rostom, R., Huang, Y. et al. (2020)
> Cardelino: computational integration of somatic clonal substructure and single-cell transcriptomes.
> Nature Methods

## About the name

`cardelino` is almost an anagram of "clone ID R" and is almost the same as the 
Italian for "goldfinch", a common and attractive European bird, pictured below
and used in `cardelino`'s hex sticker. In the Western art canon, the goldfinch 
is considered a ["saviour" bird](https://en.wikipedia.org/wiki/European_goldfinch) 
and appears in notable paintings from the [Italian renaissance](https://en.wikipedia.org/wiki/Madonna_del_cardellino)
and the [Dutch Golden Age](https://en.wikipedia.org/wiki/The_Goldfinch_(painting)). 
Perhaps this package may prove a saviour for certain single-cell datasets!

<img src=inst/cardelino_med.jpg height="200">

**Acknowledgement:**
The `cardelino` image was produced by [Darren Bellerby](https://www.flickr.com/photos/world-birds/). It was obtained from
[Flickr](https://www.flickr.com/photos/world-birds/18740373165/in/photolist-uy2j3a-uxAdib-aLcHGB-9BjDvc-YkgQg7-QN9Tr1-BVjkHh-8oWiKC-WFkDcS-nhZzXt-Y4zM2h-zULNgX-7uZCFT-f5ghc4-Ugx9pj-UJ5tog-7v4rVy-7wsLpm-bru3Ha-JnmcUQ-frkUqa-bohcgU-KAB14-dieCGY-FJ6n6A-GHJ5UK-X2qjGh-8cAjtw-FshfBi-8cwZst-qEMHSX-dTtAUs-EtqKxo-oZdJB3-8cx1Tn-D1jHjU-PWzWY2-brtKfH-ch2tvW-qEFKTd-wVmxsG-oYZbhP-Aa5cBB-h6aQf6-9Bny23-ayfnFS-dgG2Kn-QUyKgf-bBc31B-cVik3)
and is reproduced here under a CC-BY-2.0
[licence](https://creativecommons.org/licenses/by/2.0/legalcode).

# inst

This folder contains the files:

-   `CITATION`: Citation information for cardelino.
-   `cardelino_image_attribution.txt`: Attribution information for the photo for a goldfinch used in the package sticker.
-   `cardelino_sticker.png`: The cardelino's hex sticker.
-   `cardelino_med.jpg`: The photo of a goldfinch used in cardelino's sticker.

It also contains the folder `extdata`, which contains extra data used in the package's vignettes and examples.

Specifically, `extdata` contains the files:

-   `canopy_results.coveraged.rds`: Canopy tree results. Used in `vignette-cloneid.Rmd`.
-   `cells.donorid.vcf.gz`: A VCF data file from cells. Used in the examples of functions in `read_data.R`.
-   `cellSNP.cells.vcf.gz`: A VCF data file created using cellSNP. Used in the examples of functions in `read_data.R`.
-   `donors.donorid.vcf.gz`: A VCF data file from a donor. Used the in the examples of `get_snp_matrices()`.
-   `passed_ad.mtx`: AD matrix created using MQUAD. Used in the "Clone ID on mitochondrial variations" section of `vignette-cloneid.Rmd`.
-   `passed_dp.mtx`: DP matrix created using MQUAD. Used in the "Clone ID on mitochondrial variations" section of `vignette-cloneid.Rmd`.
-   `passed_variant_names.txt`: Variant names for the AD and DP matrices. Used in the "Clone ID on mitochondrial variations" section of `vignette-cloneid.Rmd`.

## ----knitr-options, echo=FALSE, message=FALSE, warning=FALSE------------------
## To render an HTML version that works nicely with github and web pages, do:
## rmarkdown::render("vignettes/vignette-cloneid.Rmd", "all")
library(knitr)
opts_chunk$set(fig.align = 'center', fig.width = 6, fig.height = 5, dev = 'png',
    warning = FALSE, error = FALSE, message = FALSE)
library(ggplot2)
library(BiocStyle)
theme_set(theme_bw(12))

## ----load-pkg-----------------------------------------------------------------
library(ggplot2)
library(cardelino)

## ----read-vcf-data------------------------------------------------------------
vcf_file <- system.file("extdata", "cellSNP.cells.vcf.gz", 
                        package = "cardelino")
input_data <- load_cellSNP_vcf(vcf_file)

## ----allele-frequency, , fig.height = 3, fig.width = 5------------------------
AF <- as.matrix(input_data$A / input_data$D)

p = pheatmap::pheatmap(AF, cluster_rows=FALSE, cluster_cols=FALSE,
                   show_rownames = TRUE, show_colnames = TRUE,
                   labels_row='77 cells', labels_col='112 SNVs',
                   angle_col=0, angle_row=0)

## ----read-canopy-data---------------------------------------------------------
canopy_res <- readRDS(system.file("extdata", "canopy_results.coveraged.rds", 
                                  package = "cardelino"))
Config <- canopy_res$tree$Z

## ----correct-variant-ids------------------------------------------------------
rownames(Config) <- gsub(":", "_", rownames(Config))

## ----plot-tre-----------------------------------------------------------------
plot_tree(canopy_res$tree, orient = "v")

## ----cell-assign--------------------------------------------------------------
set.seed(7)
assignments <- clone_id(input_data$A, input_data$D, Config = Config,
                        min_iter = 800, max_iter = 1200)
names(assignments)

## ----prob-heatmap-------------------------------------------------------------
prob_heatmap(assignments$prob)

## ----assign-cell-clone-easy---------------------------------------------------
df <- assign_cells_to_clones(assignments$prob)
head(df)
table(df$clone)

## ----Config update------------------------------------------------------------
heat_matrix(t(assignments$Config_prob - Config)) + 
  scale_fill_gradient2() +
  ggtitle('Changes of clonal Config') + 
  labs(x='Clones', y='112 SNVs') +
  theme(axis.text.y = element_blank(), legend.position = "right")

## ----results-plot, fig.height = 4.5, fig.width = 7.5--------------------------
AF <- as.matrix(input_data$A / input_data$D)
cardelino::vc_heatmap(AF, assignments$prob, Config, show_legend=TRUE)

## ----read in mtx files--------------------------------------------------------
AD_file <- system.file("extdata", "passed_ad.mtx", package = "cardelino")
DP_file <- system.file("extdata", "passed_dp.mtx", package = "cardelino")
id_file <- system.file("extdata", "passed_variant_names.txt", 
                       package = "cardelino")

AD <- Matrix::readMM(AD_file)
DP <- Matrix::readMM(DP_file)
var_ids <- read.table(id_file, )
rownames(AD) <- rownames(DP) <- var_ids[, 1]
colnames(AD) <- colnames(DP) <- paste0('Cell', seq(ncol(DP)))

## ----pheatmap, fig.height = 4.5, fig.width = 8.5------------------------------
AF_mt <- as.matrix(AD / DP)
pheatmap::pheatmap(AF_mt)

## ----denovo-mtDNA-------------------------------------------------------------
set.seed(7)
assign_mtClones <- clone_id(AD, DP, Config=NULL, n_clone = 3, 
                            keep_base_clone=FALSE)

## ----vc_heatmap-mtDNA, fig.height = 4.2, fig.width = 7------------------------
Config_mt <- assign_mtClones$Config_prob
Config_mt[Config_mt >= 0.5] = 1
Config_mt[Config_mt <  0.5] = 0
cardelino::vc_heatmap(AF_mt, assign_mtClones$prob, Config_mt, show_legend=TRUE)

## -----------------------------------------------------------------------------
sessionInfo()

## This file is for binomial distributions

#' EM algorithm for estimating binomial mixture model
#'
#'
#' @param k A vector of integers. number of success
#' @param n A vector of integers. number of trials
#' @param n_components A number. number of components
#' @param p_init A vector of floats with length n_components, the initial value
#'   of p
#' @param learn_p bool(1) or a vector of bool, whether learn each p
#' @param max_iter integer(1). number of maximum iterations
#' @param min_iter integer(1). number of minimum iterations
#' @param logLik_threshold A float. The threshold of logLikelihood increase for
#' detecting convergence
#'
#' @return a list containing \code{p}, a vector of floats between 0 and 1
#' giving the estimated success probability for each component, \code{psi},
#' estimated fraction of each component in the mixture, and \code{prob}, the
#' matrix of fitted probabilities of each observation belonging to each
#' component.
#'
#' @import stats
#'
#' @export
#'
#' @examples
#' n1 <- array(sample(1:30, 50, replace = TRUE))
#' n2 <- array(sample(1:30, 200, replace = TRUE))
#' k1 <- apply(n1, 1, rbinom, n = 1, p = 0.5)
#' k2 <- apply(n2, 1, rbinom, n = 1, p = 0.01)
#' RV <- mixBinom(c(k1, k2), c(n1, n2))
mixBinom <- function(k, n, n_components = 2, p_init = NULL, learn_p = TRUE,
    min_iter = 10, max_iter = 1000, logLik_threshold = 1e-5) {
    if (length(k) != length(n)) {
        stop("n and k must be of the same length (number of observations)\n")
    }
    S <- length(n)

    ## Random initialzation on parameters
    if (is.null(p_init)) {
        p <- stats::runif(n_components, 0.0, 1.0)
    } else {
        p <- p_init
    }
    psi <- stats::runif(n_components, 0.0, 1.0)
    psi <- psi / sum(psi)

    if (length(learn_p) == 1) {
        learn_p <- rep(learn_p, n_components)
    }

    logLik_mat <- matrix(0, nrow = S, ncol = n_components)

    ## Iterations
    for (it in seq_len(max_iter)) {
        ## E-step:
        RV <- predMixBinom(k, n, p = p, psi = psi)
        prob_mat <- RV$prob
        logLik_new <- RV$logLik

        ## M-step
        for (j in seq_len(n_components)) {
            if (learn_p[j]) {
                p[j] <- sum(prob_mat[, j] * k) / sum(prob_mat[, j] * n)
            }
            psi[j] <- sum(prob_mat[, j]) / S
        }

        ## Check convergence
        if (it > min_iter && logLik_new - logLik_old < logLik_threshold) {
            break
        } else {
            logLik_old <- logLik_new
        }
    }

    ## return values
    return_list <- list(
        "p" = p, "psi" = psi, "prob" = prob_mat,
        "logLik" = logLik_new
    )
    return_list
}


#' Predicted probability from learned binomial mixture model
#'
#' @param k A vector of integers. number of success
#' @param n A vector of integers. number of trials
#' @param p a vector of binomial success probabilities
#' @param psi A float between 0 and 1. fraction of each component
#'
#' @return A list with two components: prob, a matrix representing the 
#'   probability of each of the passed values coming from each component of the
#'   mixture and logLik, the total log-likelihood of the new samples.
#'
#' @export
#'
#' @examples
#' n1 <- array(sample(1:30, 50, replace = TRUE))
#' n2 <- array(sample(1:30, 200, replace = TRUE))
#' k1 <- apply(n1, 1, rbinom, n = 1, p = 0.5)
#' k2 <- apply(n2, 1, rbinom, n = 1, p = 0.01)
#' RV <- mixBinom(c(k1, k2), c(n1, n2))
#' RV_pred <- predMixBinom(3, 10, RV$p, RV$psi)
predMixBinom <- function(k, n, p, psi) {
    if (length(p) != length(psi)) {
        stop(
            "p and psi must be of the same length ",
            "(number of mixture components)\n"
        )
    }
    if (length(k) != length(n)) {
        stop("n and k must be of the same length (number of observations)\n")
    }

    ## assignment probability
    logLik_mat <- matrix(nrow = length(n), ncol = length(p))
    for (j in seq_len(length(p))) {
        logLik_mat[, j] <- log(psi[j]) + dbinom(k,
            size = n, prob = p[j],
            log = TRUE
        )
    }
    logLik_mat_amplify <- logLik_mat - matrixStats::rowMaxs(logLik_mat)
    prob_mat <- exp(logLik_mat_amplify) / rowSums(exp(logLik_mat_amplify))

    ## logLikelihood
    logLik_vec <- rep(NA, nrow(logLik_mat))
    for (i in seq_len(nrow(logLik_mat))) {
        logLik_vec[i] <- matrixStats::logSumExp(logLik_mat[i, ], na.rm = TRUE)
    }
    logLik_val <- sum(logLik_vec, na.rm = TRUE)

    list("prob" = prob_mat, "logLik" = logLik_val)
}

# Plotting functions

# Functions for heatmap plots

#' Plot a heatmap for probability of clone assignment
#'
#' @param prob_mat A matrix (M x K), the probability of cell j to clone k
#' @param threshold A float value, the threshold for assignable cells
#' @param mode A string, the method for defining scores for filtering cells:
#' best and delta. best: highest probability of a cell to K clones, delta: the
#' difference between the best and second.
#'
#' @param cell_idx A vector the indices of the input cells. If NULL, order by
#' the probability of each clone
#' @return a ggplot object
#'
#' @export
#'
#' @examples
#' data(example_donor)
#' assignments <- clone_id(A_clone, D_clone, Config = tree$Z, inference = "EM")
#' fig <- prob_heatmap(assignments$prob)
prob_heatmap <- function(prob_mat, threshold = 0.5, mode = "best",
    cell_idx = NULL) {
    cell_label <- cardelino::rowArgmax(prob_mat)
    prob_value <- cardelino::rowMax(prob_mat, mode = mode)
    # add clone id
    colnames(prob_mat) <- paste0("C", seq_len(ncol(prob_mat)))
    for (i in seq_len(ncol(prob_mat))) {
        conf_frac <- mean(cell_label[prob_value >= threshold] == i)
        colnames(prob_mat)[i] <- paste0(
            "C", i, ": ",
            round(conf_frac * 100, digits = 1), "%"
        )
    }

    if (is.null(cell_idx)) {
        cell_idx <- order(cell_label - diag(prob_mat[, cell_label]))
    }
    nba.m <- data.frame(
        Cell = rep(seq_len(nrow(prob_mat)), ncol(prob_mat)),
        Clone = rep(colnames(prob_mat), each = nrow(prob_mat)),
        Prob = as.vector(prob_mat[cell_idx, ]),
        stringsAsFactors = FALSE
    )

    fig_assign <- ggplot(nba.m, aes_string("Clone", "Cell", fill = "Prob")) +
        geom_tile(show.legend = TRUE) +
        scale_fill_gradient(low = "white", high = "firebrick4") +
        ylab(paste("Clonal assignment:", nrow(prob_mat), "cells")) +
        heatmap.theme()

    fig_assign
}

#' Plot heatmap from a matrix
#'
#' @param mat A matrix to show, column by x-axis and row by y-axis
#' @param base_size Numeric value for the base size in theme_bw
#' @param digits Integer value for the number of digits to show
#' @param show_value Logical value for showing the value for each element or not
#' @return A ggplot heatmap visualization of the passed matrix.
#' 
#' @examples 
#' mat <- matrix(rnorm(9), ncol = 3, nrow = 3) + diag(rnorm(3, 2, 0.1))
#' rownames(mat) <- paste0("sample_", letters[1:3])
#' colnames(mat) <- paste0("var_", 1:3)
#' heat_matrix(mat)
#' 
#' # Additional arguments. 
#' heat_matrix(mat, base_size = 6)
#' heat_matrix(mat, show_value = TRUE)
#' heat_matrix(mat, show_value = TRUE, digits = 4)
#' @export
heat_matrix <- function(mat, base_size = 12, digits = 2, show_value = FALSE) {
    df <- mtx_to_df(mat)
    if (!is.null(rownames(mat))) {
        df$Var1 <- factor(df$Var1, rownames(mat))
    }
    if (!is.null(colnames(mat))) {
        df$Var2 <- factor(df$Var2, colnames(mat))
    }

    df$value <- round(df$value, digits = digits)
    heat.plot <- ggplot(df, aes_string(x = "Var1", y = "Var2")) +
        geom_tile(aes_string(fill = "value"), colour = "grey") +
        scale_fill_gradient(low = "white", high = "steelblue") +
        # theme_grey(base_size = base_size) +
        theme_bw(base_size = base_size) +
        scale_x_discrete(expand = c(0, 0)) +
        scale_x_discrete(expand = c(0, 0)) +
        theme(
            legend.position = "none",
            panel.grid.major = element_blank(),
            panel.border = element_blank(),
            panel.background = element_blank(),
            axis.ticks.x = ggplot2::element_blank(),
            axis.ticks.y = ggplot2::element_blank()
        )
    if (show_value) {
        heat.plot <- heat.plot +
            geom_text(aes_string(label = "value"),
                vjust = 0.5, size = base_size * 0.25
            )
    }
    heat.plot
}


#' The theme of heatmaps for prob_heatmap and sites_heatmap
#'
#' @param legend.position character, describes where to place legend on plot
#' (passed to \code{\link[ggplot2]{theme_gray}})
#' @param size numeric, base font size for plot (passed to
#' \code{\link[ggplot2]{theme_gray}})
#' @return a ggplot theme based on theme_gray
#'
heatmap.theme <- function(legend.position = "bottom", size = 12) {
    ggplot2::theme_gray(base_size = size) + ggplot2::theme(
        axis.text = ggplot2::element_text(size = size),
        axis.title = ggplot2::element_text(face = "bold", size = size),
        axis.title.x = ggplot2::element_blank(),
        axis.ticks.x = ggplot2::element_blank(),
        axis.ticks.y = ggplot2::element_blank(),
        axis.text.y = ggplot2::element_blank(),
        plot.title = ggplot2::element_text(
            face = "bold", size = size * 1.3,
            hjust = 0.5
        ),
        panel.grid.major = ggplot2::element_blank(),
        panel.border = ggplot2::element_blank(),
        panel.background = ggplot2::element_blank(),
        legend.position = legend.position,
        legend.title = ggplot2::element_text(size = size * 1.1)
    )
}


#' Plot a variant-cell heatmap for cell clonal assignment
#'
#' @param mat A matrix for heatmap: N variants x M cells. row and column will be
#' sorted automatically.
#' @param prob A matrix of probability of clonal assignment: M cells x K clones
#' @param Config A binary matrix of clonal Configuration: N variants x K clones
#' @param show_legend A bool value: if TRUE, show the legend
#'
#' @return a pheatmap object
#'
#' @import pheatmap
#' @import ggplot2
#' @return a ggplot object
#'
#' @export
#'
#' @examples
#' data(example_donor)
#' assignments <- clone_id(A_clone, D_clone, Config = tree$Z)
#' fig <- vc_heatmap(assignments$prob_variant, assignments$prob, tree$Z)
#' @references
#' This function makes use of the \code{\link{pheatmap}} packages
#'
vc_heatmap <- function(mat, prob, Config, show_legend = FALSE) {
    # sort variants
    mut_label <- Config %*% (2**seq(ncol(Config), 1))
    mut_label <- seq(length(unique(mut_label)), 1)[as.factor(mut_label)]
    idx_row <- order(mut_label - rowMeans(mat, na.rm = TRUE) * 0.9 + 0.05)
    anno_row <- data.frame(Mut = as.factor(mut_label))
    row.names(anno_row) <- row.names(Config)

    # sort cells
    cell_label <- cardelino::rowArgmax(prob)
    idx_col <- order(cell_label - diag(prob[, cell_label]))
    anno_col <- data.frame(
        Clone = as.factor(cell_label),
        Prob = diag(prob[, cell_label])
    )
    row.names(anno_col) <- row.names(prob)

    # order the variant-cell matrix
    mat <- mat[idx_row, ][, idx_col]
    anno_row <- anno_row[idx_row, , drop = FALSE]
    anno_col <- anno_col[idx_col, , drop = FALSE]
    gaps_col <- cumsum(table(anno_col[[1]]))
    gaps_row <- cumsum(table(anno_row[[1]]))

    # pheatmap
    fig <- pheatmap::pheatmap(mat,
        legend = show_legend,
        cluster_rows = FALSE, cluster_cols = FALSE,
        gaps_row = gaps_row, gaps_col = gaps_col,
        annotation_row = anno_row,
        annotation_col = anno_col,
        show_rownames = FALSE, show_colnames = FALSE
    )
    fig
}


## Functions for plotting phylogenetic trees

#' Plot a phylogenetic tree
#'
#' @param tree A phylgenetic tee object of class "phylo"
#' @param orient A string for the orientation of the tree: "v" (vertical) or "h"
#' (horizontal)
#'
#' @details This function plots a phylogenetic tree from an object of class
#' "phylo", as produced, for example, by the Canopy package.
#'
#' @return a ggtree object
#'
#' @importFrom ggtree ggtree geom_tiplab
#' @import ggplot2
#'
#' @author Davis McCarthy and Yuanhua Huang
#'
#' @export
#'
#' @references
#' This function makes use of the \code{\link{ggtree}} package:
#'
#' Guangchuang Yu, David Smith, Huachen Zhu, Yi Guan, Tommy Tsan-Yuk Lam.
#' ggtree: an R package for visualization and annotation of phylogenetic trees
#' with their covariates and other associated data. Methods in Ecology and
#' Evolution 2017, 8(1):28-36, doi:10.1111/2041-210X.12628
#'
#' @examples
#' data(example_donor)
#' plot_tree(tree, orient = "v")
plot_tree <- function(tree, orient = "h") {
    node_total <- max(tree$edge)
    node_shown <- ncol(tree$Z)
    node_hidden <- node_total - node_shown
    if (!is.null(tree$P)) {
        tree$tip.label[seq_len(node_shown)] <- paste0(
            "C", seq_len(node_shown),
            ": ", round(tree$P[, 1] * 100, digits = 0), "%"
        )
    }

    mut_ids <- 0
    mut_id_all <- tree$Z %*% (2**seq(ncol(tree$Z), 1))
    mut_id_all <- seq(length(unique(mut_id_all)), 1)[as.factor(mut_id_all)]

    branch_ids <- NULL
    for (i in seq_len(node_total)) {
        mut_num <- sum(tree$sna[, 3] == i)
        if (mut_num == 0) {
            if (i == node_shown + 1) {
                branch_ids <- c(branch_ids, "Root")
            } else {
                branch_ids <- c(branch_ids, "")
            }
        } else {
            mut_ids <- mut_ids + 1
            branch_ids <- c(
                branch_ids,
                paste0("M", mut_ids, ": ", mut_num, " SNVs")
            )
        }
    }
    pt <- ggtree::ggtree(tree)
    pt <- pt + ggplot2::geom_label(ggplot2::aes_string(x = "branch"),
        label = branch_ids, color = "firebrick"
    )
    pt <- pt + ggplot2::xlim(-0, node_hidden + 0.5) +
        ggplot2::ylim(0.8, node_shown + 0.5) # the degree may not be 3
    if (orient == "v") {
        pt <- pt + ggtree::geom_tiplab(hjust = 0.39, vjust = 1.0) +
            ggplot2::scale_x_reverse() + ggplot2::coord_flip()
    } else {
        pt <- pt + ggtree::geom_tiplab(hjust = 0.0, vjust = 0.5)
    }
    pt
}


#' Define a publication-style plot theme
#'
#' @param size numeric, base font size for adapted ggplot2 theme
#' @return a ggplot theme based on theme_classic
#'
#' @details This theme modifies the \code{\link[ggplot2]{theme_classic}} theme
#' in ggplot2.
#'
#' @import ggplot2
#' @export
#' @examples
#' library(ggplot2)
#' x <- sample(10)
#' y <- x + runif(10) - 0.5
#' df <- data.frame(x = x, y = y)
#' fig <- ggplot(df, aes(x = x, y = y)) +
#'     geom_point() +
#'     pub.theme()
pub.theme <- function(size = 12) {
    theme_classic(base_size = size) +
        ggplot2::theme(
            axis.text = ggplot2::element_text(size = size),
            axis.title = ggplot2::element_text(
                face = "bold", size = size
            ),
            plot.title = ggplot2::element_text(
                size = size * 1.2, hjust = 0.5
            ),
            legend.title = ggplot2::element_text(size = size * 1.1),
            legend.text = ggplot2::element_text(size = size),
            panel.grid.major = ggplot2::element_line(
                size = 0.1, colour = "#d3d3d3"
            ),
            panel.grid.minor = ggplot2::element_line(
                size = 0.05, colour = "#d3d3d3"
            )
        )
}


#' Define a publication-style plot theme
#'
#' @param Config1 variant by clone matrix defining the first clonal structure
#' @param Config2 variant by clone matrix defining the second clonal structure
#' @param show_variant_names logical(1), should the variant names (rownames of
#' Config matrices) be shown on the plot? Default is \code{FALSE}.
#'
#' @return a ggplot heatmap style plot showing the differences between the two
#' Config matrices, specifically the differences Config1 - Config2.
#'
#' @import ggplot2
#' @export
#' @examples
#' Config1 <- matrix(c(
#'     rep(0, 15), rep(1, 8), rep(0, 7),
#'     rep(1, 5), rep(0, 3), rep(1, 7)
#' ), ncol = 3)
#' Config2 <- matrix(c(
#'     rep(0, 15), rep(1, 8), rep(1, 7),
#'     rep(0, 5), rep(1, 3), rep(1, 7)
#' ), ncol = 3)
#' rownames(Config1) <- rownames(Config2) <- paste0("var", 1:nrow(Config1))
#' colnames(Config1) <- colnames(Config2) <- paste0("clone", 1:ncol(Config1))
#' plot_config_diffs(Config1, Config2)
plot_config_diffs <- function(Config1, Config2, show_variant_names = FALSE) {
    if (!identical(rownames(Config1), rownames(Config2))) {
        stop("Config matrices must have identical rownames.")
    }
    if (!identical(colnames(Config1), colnames(Config2))) {
        stop("Config matrices must have identical colnames.")
    }
    diffs <- Config1 - Config2
    df <- data.frame(
        variant = factor(rep(rownames(Config1), ncol(Config1)),
            levels = rev(rownames(Config1))
        ),
        clone = rep(colnames(Config1), each = nrow(Config1)),
        diffs = as.vector(diffs)
    )
    p_out <- ggplot(
        df,
        aes_string(x = "clone", y = "variant", fill = "diffs")
    ) +
        geom_raster() +
        scale_x_discrete(position = "top") +
        scale_fill_gradient2(
            low = "dodgerblue4", mid = "white",
            high = "firebrick4", midpoint = 0, space = "Lab",
            na.value = "grey50", guide = "colourbar",
            aesthetics = "fill", name = "Config\ndifferences",
            limits = c(-1, 1)
        ) +
        heatmap.theme() +
        theme(
            legend.position = "bottom",
            legend.key.width = unit(0.5, "in")
        )
    if (!show_variant_names) {
        p_out <- p_out + theme(
            axis.text.y = element_blank(),
            axis.title.y = element_blank()
        )
    }
    p_out
}

# Tests for plotting functions
# library(cardelino); library(testthat); source("test-plotting.R")

context("test plotting")

data(example_donor)

test_that("plot tree works as expected", {
    fig3 <- plot_tree(tree, orient = "v")
    expect_is(fig3, "ggtree")
})

test_that("plot_config_diffs works as expected", {
    Config1 <- matrix(c(
        rep(0, 15), rep(1, 8), rep(0, 7),
        rep(1, 5), rep(0, 3), rep(1, 7)
    ), ncol = 3)
    Config2 <- matrix(c(
        rep(0, 15), rep(1, 8), rep(1, 7),
        rep(0, 5), rep(1, 3), rep(1, 7)
    ), ncol = 3)
    rownames(Config1) <- rownames(Config2) <- paste0("var", 1:nrow(Config1))
    colnames(Config1) <- colnames(Config2) <- paste0("clone", 1:ncol(Config1))
    fig4 <- plot_config_diffs(Config1, Config2) + pub.theme()
    expect_is(fig4, "ggplot")
})

# Tests for clone_id methods
# library(cardelino); library(testthat); source("test-clone_id.R")

context("test clone ID")
data("example_donor")

test_that("binomial EM inference works as expected", {
    assignments_EM <- clone_id(A_clone, D_clone,
        Config = tree$Z,
        inference = "EM"
    )
    expect_is(assignments_EM, "list")
})


assignments <- clone_id(A_clone, D_clone,
    Config = tree$Z,
    min_iter = 200, max_iter = 500,
    relax_Config = TRUE, relabel = TRUE
)

test_that("default inference works as expected", {
    expect_is(assignments, "list")
})


context("test plotting")

test_that("heatmap for assignment probability works as expected", {
    fig1 <- prob_heatmap(assignments$prob)
    expect_is(fig1, "ggplot")
})

test_that("pheatmap for variants probability across cells works as expected", {
    fig2 <- vc_heatmap(assignments$prob_variant, assignments$prob, tree$Z)
    expect_is(fig2, "pheatmap")
})

context("test assessment.R")
test_that("assign_scores, multiPRC, binaryPRC work as expected", {
    I_sim <- (assignments$prob == rowMax(assignments$prob))
    res <- assign_scores(assignments$prob, I_sim)
    expect_is(res, "list")
})

test_that("binaryROC work as expected", {
    I_sim <- (assignments$prob == rowMax(assignments$prob))
    res <- binaryROC(assignments$prob[, 1], I_sim[, 1])
    expect_is(res, "list")
})

#' Get a clonal tree from a configuration matrix
#'
#' @param Config variant x clone matrix of binary values. The clone-variant
#' configuration, which encodes the phylogenetic tree structure. This is the
#' output Z of Canopy
#' @param P a one-column numeric matrix encoding the (observed or estimated)
#' prevalence (or frequency) of each clone
#' @param strictness character(1), a character string defining the strictness of
#' the function if there are all-zero rows in the Config matrix. If \code{"lax"}
#' then the function silently drops all-zero rows and proceeds. If \code{"warn"}
#' then the function warns of dropping all-zero rows and proceeds. If
#' \code{"error"} then the function throws an error is all-zero rows are
#' detected.
#'
#' @return
#' An object of class "phylo" describing the tree structure. The output object
#' also contains an element "sna" defining the clustering of variants onto the
#' branches of the tree, and if \code{P} is non-null it also contains VAF
#' (variant allele frequency), CCF (cell clone fraction) and clone prevalence
#' values (computed from the supplied \code{P} argument).
#'
#' @details
#' Output tree may be nonsensical if the input \code{Config} matrix does not
#' define a coherent tree structure.
#'
#' @author Davis McCarthy
#'
#' @import utils
#' @export
#'
#' @examples
#' Configk3 <- matrix(c(
#'     rep(0, 15), rep(1, 8), rep(0, 7), rep(1, 5), rep(0, 3),
#'     rep(1, 7)
#' ), ncol = 3)
#' tree_k3 <- get_tree(Config = Configk3, P = matrix(rep(1 / 3, 3), ncol = 1))
#' plot_tree(tree_k3)
get_tree <- function(Config, P = NULL, strictness = "lax") {
    if (!is.null(P)) {
        if (ncol(P) != 1) {
            stop(
                "P must be a matrix with one column encoding clone ",
                "prevalence values"
            )
        }
    }
    all_zero_rows <- rowSums(Config) == 0
    strictness <- match.arg(strictness, c("lax", "warn", "error"))
    if (any(all_zero_rows)) {
        if (strictness == "error") {
            stop("Config matrix contains all-zero rows.")
        } else {
            if (strictness == "warn") {
                warning(
                    "Dropped ", sum(all_zero_rows),
                    " all-zero rows from Config matrix."
                )
            } else {
                message(
                    "Dropped ", sum(all_zero_rows),
                    " all-zero rows from Config matrix."
                )
            }
            Config <- Config[!all_zero_rows, ]
        }
    }
    k <- ncol(Config) # number of clones
    varnames <- rownames(Config)
    sna <- matrix(nrow = nrow(Config), ncol = 3)
    sna[, 1] <- seq_len(nrow(sna))
    rownames(sna) <- varnames
    colnames(sna) <- c("sna", "sna.st.node", "sna.ed.node")
    tip_label <- seq_len(k)
    tip_vals <- 2^seq_len(k)
    ## Need to determine number of internal nodes in the tree
    Config2 <- t(t(Config) * tip_vals)
    var_bin_vals <- rowSums(Config2)
    node_vals <- names(table(var_bin_vals))
    node_vals <- node_vals[!(node_vals %in% tip_vals)]
    node_num <- sum(!(node_vals %in% tip_vals))
    ## define a list with subsets of edge matrices
    ## start with root node (k+1), which always connects to tip 1 (base clone)
    if (node_num > 0.5) {
        node_def_list <- list()
        edge_list <- list()
        for (i in seq_len(k - 1)) {
            clone_combos <- utils::combn(2:k, (k - i), simplify = FALSE)
            for (j in seq_len(length(clone_combos))) {
                test_sum <- sum(tip_vals[clone_combos[[j]]])
                if (test_sum %in% node_vals) {
                    node_def_list[[
                    paste0("node", paste0(clone_combos[[j]]), collapse = "_")
                    ]] <-
                        clone_combos[[j]]
                }
            }
        }
        if (node_num != length(node_def_list)) {
            stop("Conflict in computed number of internal nodes.")
        }
        ## Sort out edges for the root node
        tip_nodes <- seq_len(k)
        root_to_tip <- tip_nodes[
            !(tip_nodes %in% unique(unlist(node_def_list)))
        ]
        edge_list[["root_node_tips"]] <- matrix(
            c(rep(k + 1, length(root_to_tip)), root_to_tip),
            nrow = length(root_to_tip)
        )
        el_counter <- 1
        for (i in seq_len(length(node_def_list))) {
            ## add edge from root to internal node if not already done
            if (i < 1.5) {
                el_counter <- el_counter + 1
                edge_list[[el_counter]] <- matrix(c(k + 1, k + 1 + i), nrow = 1)
                sna[var_bin_vals == sum(2^node_def_list[[i]]), 2] <- k + 1
                sna[var_bin_vals == sum(2^node_def_list[[i]]), 3] <- k + 1 + i
            } else {
                clones_in_this_node <- node_def_list[[i]]
                clones_in_prev_nodes <- unique(
                    unlist(node_def_list[seq_len(i - 1)])
                )
                if (!any(clones_in_this_node %in% clones_in_prev_nodes)) {
                    el_counter <- el_counter + 1
                    edge_list[[el_counter]] <- matrix(c(k + 1, k + 1 + i),
                        nrow = 1
                    )
                    sna[var_bin_vals == sum(2^node_def_list[[i]]), 2] <- k + 1
                    sna[var_bin_vals == sum(2^node_def_list[[i]]), 3] <-
                        k + 1 + i
                }
            }
            ## add edge from internal node to internal node
            ## if all of the clones for the node are present in the previous
            ## node in the tree (immediately above in the hierarchy), then add
            ## the edge check the size of previous nodes, and select the node
            ## that has minimum number of clones that is more than the number
            ## in this node
            if (i > 1.5) {
                prev_nodes <- seq_len(i - 1)
                prev_node_sizes <- vapply(
                    node_def_list[prev_nodes], length,
                    numeric(1)
                )
                prev_nodes <- prev_nodes[prev_node_sizes >
                    length(node_def_list[[i]])]
                min_prev_node_size <- min(prev_node_sizes[prev_nodes])
                prev_nodes <- prev_nodes[prev_node_sizes[prev_nodes] ==
                    min_prev_node_size]
                for (j in prev_nodes) {
                    if (all(node_def_list[[i]] %in% node_def_list[[j]])) {
                        el_counter <- el_counter + 1
                        edge_list[[el_counter]] <- matrix(
                            c(k + 1 + j, k + 1 + i),
                            nrow = 1
                        )
                        sna[var_bin_vals == sum(2^node_def_list[[i]]), 2] <-
                            k + 1 + j
                        sna[var_bin_vals == sum(2^node_def_list[[i]]), 3] <-
                            k + 1 + i
                    }
                }
            }
            ## add edge from internal node to tip
            ## (if clone not present in any subsequent nodes)
            if (node_num < 1.5) {
                ## if only one internal node, there are edges from this node to
                ## all tips
                node_to_tip <- tip_nodes[-1]
                el_counter <- el_counter + 1
                edge_list[[el_counter]] <- matrix(
                    c(rep(k + 1 + i, length(node_to_tip)), node_to_tip),
                    nrow = length(node_to_tip)
                )
                for (m in node_to_tip) {
                    sna[var_bin_vals == sum(2^m), 2] <- k + 1 + i
                    sna[var_bin_vals == sum(2^m), 3] <- m
                }
            } else {
                ## if more than one internal node, need to check if tips
                ## mentioned in this node appear in any subsequent nodes
                node_to_tip <- node_def_list[[i]]
                if (i < node_num) {
                    node_to_tip <- node_to_tip[
                        !(node_to_tip %in% 
                            unique(unlist(node_def_list[(i + 1):node_num])))
                    ]
                }
                ## else this is the last node; just connect edges from node to
                ## tips
                if (length(node_to_tip) > 0.5) {
                    el_counter <- el_counter + 1
                    edge_list[[el_counter]] <- matrix(
                        c(rep(k + 1 + i, length(node_to_tip)), node_to_tip),
                        nrow = length(node_to_tip)
                    )
                    for (m in node_to_tip) {
                        sna[var_bin_vals == sum(2^m), 2] <- k + 1 + i
                        sna[var_bin_vals == sum(2^m), 3] <- m
                    }
                }
            }
        }
    } else {
        edge_list <- list("root_node" = matrix(c(rep(k + 1, k), seq_len(k)),
            ncol = 2
        ))
        for (j in 2:k) {
            sna[var_bin_vals == 2^j, 2] <- k + 1
            sna[var_bin_vals == 2^j, 3] <- j
        }
    }
    # node_def_list
    edge_mat <- do.call(rbind, edge_list)
    tree_out <- list(
        edge = edge_mat, Nnode = node_num + 1,
        tip.label = tip_label
    )
    class(tree_out) <- "phylo"
    tree_out$Z <- Config
    if (!is.null(P)) {
        tree_out$P <- P
        tree_out$VAF <- tree_out$Z %*% tree_out$P / 2
        tree_out$CCF <- tree_out$Z %*% tree_out$P
    }
    tree_out$sna <- sna
    tree_out
}

#' Synthetic reads generator for genetic variants
#'
#' There are following steps to generate the simulated reads counts for variants
#' in single cells:
#' 1) given the clonal genotype and the clonal prevalence, the genotypes (i.e,
#' the clone) of cells will be generated following a multinomial distribution.
#' Note, one cell may contain variants from two clones when it is a doublet.
#' 2) given the distribution of reads coverage, e.g., a matrix of read coverage
#' from real data, (variant specific), the total reads of each variant will be
#' generated by random sampling. Note, the missing rate is governed by this
#' matrix.
#' 3) the allelic frequency of each variant will be generated by following a
#' beta distribution with parameters of mean and variance.
#' 4) Given the genotype of a cell, if the mutation exists in a cell, the
#' alteration read counts will be generated by a binomial distribution,
#' parameterized the allelic frequency, sampled from step 3.
#' 5) Given the genotype of a cell, if the mutation does not exist in a cell,
#' the alteration read counts will be generated by a binomial distribution,
#' parameterized by the technical error rate.
#'
#' @export
#'
#' @param Config A matrix of binary values. The clone-variant configuration,
#' which encodes the phylogenetic tree structure, and the genotype of each clone
#' @param D A matrix of integers. Sequencing depth for N variants across x
#' cells (ideally >100 cells). NA means 0 here.
#' @param Psi A vector of float. The fractions of each clone. If NULL, set a
#' uniform distribution.
#' @param means A vector of two floats. The mean theta_1 (false positive rate)
#' and the mean theta_2 (true positive rate).
#' @param vars A vector of two floats. The variance of theta_1 and theta_2.
#' @param cell_num A integer. The number of cells to generate.
#' @param permute_D A Boolean value. If True permute variants in D.
#' @param wise0 A string, the beta-binomial parameter specificity for theta0:
#' global, variant, element.
#' @param wise1 A string, the beta-binomial parameter specificity for theta1:
#' global, variant, element.
#' @param doublet A float between 0 and 1, the rate of doublets
#' @param sample_cell A Boolean value. If True and M > ncol(D), sample cells.
#'
#' @return a list containing \code{A_sim}, a matrix for alteration reads,
#' \code{A_sim}, a matrix for total reads, \code{I_sim}, a matrix for clonal
#' label, \code{H_sim}, a matrix for genotype, \code{theta0}, a matrix of
#' expected false positive rate, \code{theta1}, a matrix of expected true
#' positive rate, \code{theta0_binom}, theta0 as binomial parameter,
#' \code{theta1_binom}, theta0 as binomial parameter, and \code{is_doublet}, a
#' vector of Boolean value if a cell is a doublet
#'
#' @examples
#' data(simulation_input)
#' D2 <- sample_seq_depth(D_input, n_cells = 500, n_sites = nrow(tree_4clone$Z))
#' simu <- sim_read_count(tree_4clone$Z, D2, Psi = NULL, cell_num = 500)
sim_read_count <- function(Config, D, Psi = NULL,
    means = c(0.002, 0.45), vars = c(100, 1),
    wise0 = "element", wise1 = "variant", cell_num = 300,
    permute_D = FALSE, sample_cell = TRUE, doublet = 0.0) {
    M <- cell_num # number of cells
    K <- ncol(Config) # number of clones
    N <- nrow(Config) # number of variants

    shape1s <- means * vars
    shape2s <- (1.0 - means) * vars

    D[which(D == 0)] <- NA
    if (permute_D == TRUE) {
        D <- D[sample(nrow(D)), ]
    }
    if (sample_cell || M > ncol(D)) {
        D_sim <- D[, sample(ncol(D), M, replace = TRUE)]
    } else {
        D_sim <- D[, seq_len(M)]
    }

    # genotype for cells H_sim, and clone labels I_sim
    if (is.null(Psi)) {
        Psi <- rep(1 / K, K)
    }
    is_doublet <- rep(FALSE, M)
    is_doublet[sample(M, round(doublet * M))] <- TRUE
    I_sim <- rmultinom(M, 1, prob = Psi) # Clonal label for each cell
    I_sim[, is_doublet] <- rmultinom(sum(is_doublet), 2, prob = Psi)
    I_sim[which(I_sim > 1)] <- 1
    H_sim <- Config %*% I_sim # Genotype for each cell

    # generate p, D and A
    p_sim <- matrix(0, 2) # p1 and p2 for False positive and True positive
    A_sim <- matrix(NA, N, M) # Alteration counts matrix
    A_germ_sim <- matrix(NA, N, M) # Alteration counts matrix for germline var
    D_germ_sim <- matrix(NA, N, M)
    theta0_Bern_sim <- matrix(NA, N, M) # theta0 matrix
    theta1_Bern_sim <- matrix(NA, N, M) # theta1 matrix
    theta0_binom_sim <- matrix(NA, N, M) # theta0 matrix
    theta1_binom_sim <- matrix(NA, N, M) # theta1 matrix

    p_sim[1] <- stats::rbeta(1, shape1s[1], shape2s[1])
    p_sim[2] <- stats::rbeta(1, shape1s[2], shape2s[2])
    for (i in seq_len(N)) {
        if (wise0 == "variant") {
            p_sim[1] <- stats::rbeta(1, shape1s[1], shape2s[1])
        }
        if (wise1 == "variant") {
            p_sim[2] <- stats::rbeta(1, shape1s[2], shape2s[2])
        }
        for (j in seq_len(M)) {
            if (wise0 == "element") {
                p_sim[1] <- stats::rbeta(1, shape1s[1], shape2s[1])
            }
            if (wise1 == "element") {
                p_sim[2] <- stats::rbeta(1, shape1s[2], shape2s[2])
            }
            if (!is.na(D_sim[i, j])) {
                D_germ_sim[i, j] <- D_sim[i, j]
                A_germ_sim[i, j] <- rbinom(1, D_germ_sim[i, j], p_sim[2])

                theta0_binom_sim[i, j] <- p_sim[1]
                theta1_binom_sim[i, j] <- p_sim[2]
                theta0_Bern_sim[i, j] <- 1 - dbinom(0,
                    size = D_sim[i, j],
                    prob = p_sim[1]
                )
                theta1_Bern_sim[i, j] <- 1 - dbinom(0,
                    size = D_sim[i, j],
                    prob = p_sim[2]
                )
                if (H_sim[i, j] == 0) {
                    A_sim[i, j] <- rbinom(1, D_sim[i, j], p_sim[1])
                } else {
                    A_sim[i, j] <- rbinom(1, D_sim[i, j], p_sim[2])
                }
            }
        }
    }
    I_sim <- t(I_sim)
    colnames(I_sim) <- colnames(Config)
    row.names(A_sim) <- row.names(D_sim) <- row.names(Config)
    row.names(A_germ_sim) <- row.names(D_germ_sim) <- row.names(Config)

    cell_names <- paste0("cell", seq_len(ncol(A_sim)))
    row.names(I_sim) <- colnames(A_sim) <- colnames(D_sim) <- cell_names
    colnames(A_germ_sim) <- colnames(D_germ_sim) <- cell_names

    return_list <- list(
        "H_sim" = H_sim, "I_sim" = I_sim,
        "A_sim" = A_sim, "D_sim" = D_sim,
        "A_germ_sim" = A_germ_sim, "D_germ_sim" = D_germ_sim,
        "theta0" = theta0_Bern_sim, "theta1" = theta1_Bern_sim,
        "theta0_binom" = theta0_binom_sim,
        "theta1_binom" = theta1_binom_sim,
        "is_doublet" = is_doublet
    )
    return_list
}

#' Update matrix D with manually selected missing rate
#'
#' Given missing rate, the NA will be generated first. For none NA element,
#' sequencing depth with uniformly sampled from D, row wisely. Namely, the depth
#' is variant specific.
#'
#' @param D A matrix (N variants x M cells), the original sequencing coverage,
#' NA means missing
#' @param n_cells A integer, the number of the cells to generate
#' @param n_sites A integer, the number of variants to generate
#' @param missing_rate A float value, if NULL, use the same missing rate as D
#' @return a n_sites by n_cells matrix sampled from input D.
#'
#' @export
#' @examples
#' data(simulation_input)
#' D1 <- sample_seq_depth(D_input,
#'     n_cells = 500, n_sites = 50,
#'     missing_rate = 0.85
#' )
sample_seq_depth <- function(D, n_cells = NULL, n_sites = NULL,
    missing_rate = NULL) {
    D[which(D == 0)] <- NA
    if (is.null(n_cells)) {
        n_cells <- ncol(D)
    }
    if (is.null(n_sites)) {
        n_sites <- nrow(D)
    }
    missing_vector <- rowMeans(is.na(D))
    if (!is.null(missing_rate)) {
        if (missing_rate >= 1.0) {
            stop("missing rate should be < 1.0!")
        }
        m_upbound <- 1 - 1.0 / n_cells
        m_mean <- mean(missing_vector)
        amplify <- min(
            missing_rate / m_mean,
            (m_upbound - missing_rate) / (max(missing_vector) - m_mean)
        )
        missing_vector <- amplify * (missing_vector - m_mean) + missing_rate
    }

    idx <- sample(nrow(D), size = n_sites, replace = (n_sites > nrow(D)))
    D_input <- D[idx, ]
    missing_vector <- missing_vector[idx]

    D_output <- matrix(NA, nrow = n_sites, ncol = n_cells)
    for (i in seq_len(nrow(D_input))) {
        D_no_na <- D_input[i, which(!is.na(D_input[i, ]) & D_input[i, ] > 0)]
        D_output[i, ] <- sample(D_no_na, n_cells, replace = TRUE)
        missing_idx <- sample(n_cells, round(missing_vector[i] * n_cells),
            replace = FALSE
        )
        D_output[i, missing_idx] <- NA
    }

    D_output
}


#' Down sample number of SNVs in the tree
#'
#' @param tree A tree object from Canopy
#' @param n_SNV A integer, the number of SNVs to keep in the output tree
#' @return a phylo tree with down sampled variants
#'
#' @export
#' @examples
#' data(simulation_input)
#' tree_lite <- sample_tree_SNV(tree_4clone, n_SNV = 10)
sample_tree_SNV <- function(tree, n_SNV = NULL) {
    total_SNV <- nrow(tree$Z)
    if (is.null(n_SNV)) {
        n_SNV <- total_SNV
    }
    if (total_SNV < n_SNV) {
        stop("Only support down sample SNVs")
    }
    idx <- sample(total_SNV, size = n_SNV)

    out.tree <- tree
    out.tree$Z <- out.tree$Z[idx, ]
    out.tree$CCF <- out.tree$CCF[idx, , drop = FALSE]
    out.tree$VAF <- out.tree$VAF[idx, , drop = FALSE]
    out.tree$sna <- out.tree$sna[idx, , drop = FALSE]

    out.tree
}

#' Reads simulator for donor identification
#' @param GT Variant-by-donor matrix for genotypes
#' @param D_seed Variant-by-cell matrix for read coverage for generating depth,
#' which be row sample and column sample both with replacement
#' @param sample_variants logical(1), if TRUE, sample variants with replacement
#' to the same size, otherwise not
#' @param donor_size Vector of float for the fractions of each donor; default
#' NULL means uniform
#' @param beta_shapes A 3-by-2 matrix of beta parameters for genotypes: 0, 1,
#' and 2; default NULL means matrix(c(0.2, 0.5, 99.8, 99.8, 0.5, 0.2), nrow = 3)
#' @param n_cell An integer for number of total cells
#' @param doublet_rate A float from 0 to 1 for doublet rate; default NULL means
#' rate n_cell / 100000
#' 
#' @return A list of various components of the simulated dataset.
#' @export
#'
donor_read_simulator <- function(GT, D_seed, sample_variants = FALSE,
    donor_size = NULL, beta_shapes = NULL,
    n_cell = 5000, doublet_rate = NULL) {
    K <- ncol(GT) # number of clones
    N <- nrow(GT) # number of variants
    if (is.null(beta_shapes)) {
        # beta_shapes <- matrix(c(0.3, 3, 29.7, 29.7, 3, 0.3), nrow = 3)
        beta_shapes <- matrix(c(0.2, 0.1, 99.8, 99.8, 0.1, 0.2), nrow = 3)
    }
    if (is.null(doublet_rate)) {
        doublet_rate <- n_cell / 100000
        print(paste("doublet rate:", doublet_rate))
    }
    if (is.null(donor_size)) {
        donor_size <- rep(1 / K, K)
    }

    ## generate donor id matrix
    n_doublet <- round(doublet_rate * n_cell)
    I_sim <- t(rmultinom(n_cell + n_doublet, 1, prob = donor_size))

    ## generate D
    if (sample_variants) {
        D_sim <- D_seed[sample(nrow(D_seed), N, replace = TRUE), ]
    } else {
        D_sim <- D_seed
    }
    D_sim <- D_sim[, sample(ncol(D_sim), nrow(I_sim), replace = TRUE)]
    D_sim <- Matrix::Matrix(D_sim, sparse = TRUE)

    ## generate A with beta binomial
    A_sim <- D_sim
    H_sim <- GT %*% t(I_sim)
    for (ii in seq_len(3)) {
        val2 <- beta_shapes[ii, 1] + beta_shapes[ii, 2]
        val1 <- beta_shapes[ii, 1] / val2

        idx <- which(as.matrix(D_sim) > 0 & H_sim == (ii - 1))
        A_sim[idx] <- VGAM::rbetabinom(length(idx),
            size = D_sim[idx],
            prob = val1, rho = 1 / (1 + val2)
        )
    }

    ## pooling doublet
    idx_doublet1 <- sample(n_cell, n_doublet)
    idx_doublet2 <- seq(n_cell + 1, n_cell + n_doublet)
    A_sim[, idx_doublet1] <- A_sim[, idx_doublet1] + A_sim[, idx_doublet2]
    D_sim[, idx_doublet1] <- D_sim[, idx_doublet1] + D_sim[, idx_doublet2]
    I_sim[idx_doublet1, ] <- I_sim[idx_doublet1, ] + I_sim[idx_doublet2, ]
    A_sim <- A_sim[, seq_len(n_cell)]
    D_sim <- D_sim[, seq_len(n_cell)]
    I_sim <- I_sim[seq_len(n_cell), ]

    ## output
    colnames(I_sim) <- colnames(GT)
    row.names(A_sim) <- row.names(D_sim) <- row.names(GT)
    cell_names <- paste0("cell", seq_len(ncol(A_sim)))
    row.names(I_sim) <- colnames(A_sim) <- colnames(D_sim) <- cell_names

    return_list <- list(
        "A" = A_sim, "D" = D_sim,
        "I_sim" = I_sim, "GT" = GT
    )
    return_list
}

# Tests for simulation methods
# library(cardelino); library(testthat); source("test-simulator.R")

context("test simulator")

data(simulation_input)

test_that("depths generator works as expected", {
    D1 <- sample_seq_depth(D_input, n_cells = 500, n_sites = 50, missing_rate = 0.85)
    expect_is(D1, "matrix")
})

test_that("read counts generator works as expected", {
    D2 <- sample_seq_depth(D_input, n_cells = 500, n_sites = nrow(tree_4clone$Z))
    simu <- sim_read_count(tree_4clone$Z, D2, Psi = NULL, cell_num = 500)
    expect_is(simu, "list")
})

test_that("down sample variants in tree works as expected", {
    tree_lite <- sample_tree_SNV(tree_4clone, n_SNV = 10)
    expect_is(tree_lite, "phylo")
})

# A few basic functions for assessment

#' Convert a matrix to data frame
#' @param X A matrix of values
#' @return A data.frame version of the passed matrix.
#' @export
#' @examples
#' mtx_to_df(matrix(seq(12), nrow = 3))
mtx_to_df <- function(X) {
    if (is.null(row.names(X))) {
        row.names(X) <- seq(nrow(X))
    }
    if (is.null(colnames(X))) {
        colnames(X) <- seq(ncol(X))
    }
    data.frame(
        Var1 = rep(row.names(X), ncol(X)),
        Var2 = rep(colnames(X), each = nrow(X)),
        value = c(X)
    )
}

#' Maximum value for each row in a matrix
#'
#' @param X A matrix of floats.
#' @param mode A string, the method for defining scores for filtering cells:
#' best, second and delta. \code{best}: highest value for each row, similarly
#' for the \code{second}. \code{delta} is the difference between the best and
#' the second.
#' @return a vector of the collapsed value for each row, depending
#' on the mode used.
#'
#' @export
#' @examples
#' matA <- matrix(sample(seq(12)), nrow = 3)
#' rowMax(matA)
rowMax <- get_prob_value <- function(X, mode = "best") {
    max_val <- rep(0, nrow(X))
    for (i in seq_len(nrow(X))) {
        sorted_val <- sort(X[i, ], decreasing = TRUE)
        if (mode == "delta") {
            max_val[i] <- sorted_val[1] - sorted_val[2]
        } else if (mode == "second") {
            max_val[i] <- sorted_val[2]
        } else { # default mode: best
            max_val[i] <- sorted_val[1]
        }
    }
    max_val
}


#' Column index of the maximum value for each row in a matrix
#'
#' @param X A matrix of floats.
#' @return a vector of the index of column for each row. Note, when multiple
#' columns have the same value, only the earliest column will be
#' returned.
#'
#' @export
#' @examples
#' matA <- matrix(sample(seq(12)), nrow = 3)
#' rowArgmax(matA)
rowArgmax <- get_prob_label <- function(X) {
    max_idx <- rep(0, nrow(X))
    for (j in seq_len(nrow(X))) {
        max_idx[j] <- which.max(X[j, ])
    }
    max_idx
}

#' Column match between two matrices by minimum mean absolute difference
#' @param A The first matrix which will be matched
#' @param B The second matrix, the return index will be used on
#' @param force bool(1), If TRUE, force traversing all permutations of B to
#' find the optimised match to A with computing cost of O(n!). Otherwise, use
#' greedy search with computing cost of O(n^2).
#' @return \code{idx}, the column index of B to be matched to A
#' 
#' @importFrom combinat permn
#' @export
#' @examples
#' matA <- matrix(sample(seq(12)), nrow = 3)
#' col_idx <- sample(4)
#' matB <- matA[, col_idx]
#' colMatch(matB, matA)
colMatch <- function(A, B, force = FALSE) {
    if (nrow(A) != nrow(B)) {
        stop("A and B have different rows.")
    }
    if (ncol(A) > ncol(B)) {
        stop("A must have equal or smaller columns than B.")
    }
    if (force == TRUE) {
        idx_list <- combinat::permn(ncol(B))
        MAE_list <- rep(NA, length(idx_list))
        for (ii in seq_len(length(idx_list))) {
            MAE_list[ii] <- mean(abs(A - B[, idx_list[[ii]][seq_len(ncol(A))]]))
        }
        idx_out <- idx_list[[which.min(MAE_list)]][seq_len(ncol(A))]
    } else {
        MAE_mat <- matrix(0, ncol(A), ncol(B))
        for (i in seq_len(ncol(A))) {
            for (j in seq_len(ncol(B))) {
                MAE_mat[i, j] <- mean(abs(A[, i] - B[, j]))
            }
        }
        MAE_tmp <- MAE_mat
        idx_out <- rep(-1, ncol(A))
        while (-1 %in% idx_out) {
            idx_tmp <- which(MAE_tmp == min(MAE_tmp), arr.ind = TRUE)
            idx_row <- idx_tmp[1]
            idx_col <- idx_tmp[2]

            idx_out[idx_row] <- idx_col
            MAE_tmp[idx_row, ] <- max(MAE_mat) + 1
            MAE_tmp[, idx_col] <- max(MAE_mat) + 1
        }
    }
    idx_out
}


#' Precision-recall curve for binary label prediction
#' @param scores Prediction score for each sample
#' @param labels True labels for each sample, e.g., from simulation
#' @param cutoff A vector of cutoffs; if NULL use all unique scores
#' @param cut_direction A string to compare with cutoff: >=, >, <=, <
#' @param add_cut1 Logical value; if True, manually add a cutoff of 1
#' @param empty_precision Float value for default precision if no any recall
#' @return A data.frame containing recall and precision values at various 
#' cutoffs.
#' @export
#' 
#' @examples
#' scores <- 1:10
#' labels <- c(0, 0, 0, 1, 0, 1, 0, 1, 1, 1)
#' binaryPRC(scores, labels)
#' 
#' # Extra arguments. 
#' binaryPRC(scores, labels, cutoff = seq(1, 10, by = 2))
#' binaryPRC(scores, labels, cut_direction = ">")
#' binaryPRC(scores, labels, add_cut1 = TRUE)
#'
binaryPRC <- function(scores, labels, cutoff = NULL, cut_direction = ">=",
    add_cut1 = FALSE, empty_precision = 1) {
    if (is.null(cutoff)) {
        cutoff <- sort(unique(scores))
    }
    n_positive <- sum(labels == 1 | labels)

    Precision <- rep(0, length(cutoff))
    Recall <- rep(0, length(cutoff))
    for (i in seq_len(length(cutoff))) {
        if (cut_direction == "<=") {
            idx <- scores <= cutoff[i]
        } else if (cut_direction == "<") {
            idx <- scores < cutoff[i]
        } else if (cut_direction == ">") {
            idx <- scores > cutoff[i]
        } else {
            idx <- scores >= cutoff[i]
        }

        is_idx_true <- (labels[idx] == 1 | labels[idx])
        Recall[i] <- sum(is_idx_true) / n_positive
        Precision[i] <- mean(is_idx_true)
    }
    if (!is.null(empty_precision)) {
        Precision[Recall == 0] <- empty_precision
    }
    if (add_cut1) {
        cutoff <- c(cutoff, 1.0)
        Recall <- c(Recall, 0.0)
        Precision <- c(Precision, 1.0)
    }

    AUC <- 0.0
    for (i in seq_len(length(cutoff) - 1)) {
        AUC <- ((Recall[i] - Recall[i + 1]) *
            (Precision[i] + Precision[i + 1]) * 0.5 + AUC)
    }
    AUC <- AUC / (Recall[1] - Recall[length(Recall)])

    df <- data.frame(
        "cutoff" = cutoff, "Recall" = Recall,
        "Precision" = Precision
    )
    list("df" = df, "AUC" = AUC)
}

#' ROC curve for binary label prediction
#' @param scores Prediction score for each sample
#' @param labels True labels for each sample, e.g., from simulation
#' @param cutoff A vector of cutoffs; if NULL use all unique scores
#' @param cut_direction A string to compare with cutoff: >=, >, <=, <
#' @param add_cut1 Logical value; if True, manually add a cutoff of 1
#' @param cutoff_point Numeric value; additional cutoff value
#' @return A data.frame containing AUC and AUPRC at various cutoffs.
#' @export
#' 
#' 
#' @examples
#' scores <- 1:10
#' labels <- c(0, 0, 0, 1, 0, 1, 0, 1, 1, 1)
#' binaryROC(scores, labels)
#' 
#' # Extra arguments. 
#' binaryROC(scores, labels, cutoff = seq(1, 10, by = 2))
#' binaryROC(scores, labels, cut_direction = ">")
#' binaryROC(scores, labels, add_cut1 = TRUE)
#'
binaryROC <- function(scores, labels, cutoff = NULL, cut_direction = ">=",
    add_cut1 = TRUE, cutoff_point = 0.9) {
    if (is.null(cutoff)) {
        cutoff <- sort(unique(scores))
    }
    cutoff <- sort(unique(c(cutoff, cutoff_point, 0, 1)))

    TPR <- rep(0, length(cutoff))
    FPR <- rep(0, length(cutoff))
    Precision <- rep(0, length(cutoff))
    for (i in seq_len(length(cutoff))) {
        if (cut_direction == "<=") {
            idx <- scores <= cutoff[i]
        } else if (cut_direction == "<") {
            idx <- scores < cutoff[i]
        } else if (cut_direction == ">") {
            idx <- scores > cutoff[i]
        } else {
            idx <- scores >= cutoff[i]
        }

        FPR[i] <- sum(labels[idx] == 0) / sum(labels == 0) ## FPR
        TPR[i] <- sum(labels[idx] == 1) / sum(labels == 1) ## TPR
        Precision[i] <- mean(labels[idx] == 1)
    }
    Precision[(TPR == 0) & is.na(Precision)] <- 1.0

    if (add_cut1) {
        cutoff <- c(cutoff, 1.0)
        FPR <- c(FPR, 0.0)
        TPR <- c(TPR, 0.0)
        Precision <- c(Precision, 1.0)
    }

    AUC <- AUPRC <- 0.0
    for (i in seq_len(length(cutoff) - 1)) {
        AUC <- ((FPR[i] - FPR[i + 1]) *
            (TPR[i] + TPR[i + 1]) * 0.5 + AUC)
        AUPRC <- ((TPR[i] - TPR[i + 1]) *
            (Precision[i] + Precision[i + 1]) * 0.5 + AUPRC)
    }
    AUC <- AUC / (FPR[1] - FPR[length(FPR)])
    AUPRC <- AUPRC / (TPR[1] - TPR[length(TPR)])

    df <- data.frame(
        "cutoff" = cutoff, "TPR" = TPR,
        "FPR" = FPR, Precision = Precision
    )
    list("df" = df, "AUC" = AUC, "AUPRC" = AUPRC)
}


#' Precision-recall curve for multi-class prediction
#' @param prob_mat Probability matrix for each cell to each component
#' @param simu_mat The true identity of assignment from simulation
#' @param marginal_mode A string for the mode to marginalize the column: best,
#' second, or delta
#' @param cutoff A list of cutoff; if NULL use all unique scores
#' @param add_cut1 Logical value; if True, manually add a cutoff of 1
#' @param multiLabel.rm Logical value; if True, remove the samples with
#' multiple labels
#' 
#' @return A list with two components: df, a data.frame containing precision 
#' and recall values at various cutoffs and AUC, the overall AUC.
#' 
#' @export
#'
multiPRC <- function(prob_mat, simu_mat, marginal_mode = "best",
    cutoff = NULL, multiLabel.rm = TRUE, add_cut1 = FALSE) {
    if (nrow(prob_mat) != nrow(simu_mat) ||
        ncol(prob_mat) != ncol(simu_mat)) {
        stop("The shapes are not matched: prob_mat, simu_mat.")
    }

    idx_multiLabel <- rowSums(simu_mat > 0) > 1
    if (sum(idx_multiLabel)) {
        print(paste(sum(idx_multiLabel), "samples have multiple labels."))
    }

    assign_0 <- rowArgmax(simu_mat)
    assign_1 <- rowArgmax(prob_mat)
    assign_0[idx_multiLabel] <- -1

    if (marginal_mode == "sum") {
        prob_val <- rowSums(prob_mat)
    } else {
        prob_val <- rowMax(prob_mat, mode = marginal_mode)
    }

    if (multiLabel.rm) {
        prob_val <- prob_val[assign_0 > 0]
        assign_1 <- assign_1[assign_0 > 0]
        assign_0 <- assign_0[assign_0 > 0]
    }

    if (is.null(cutoff)) {
        cutoff <- sort(unique(prob_val))
    }

    Precision <- rep(0, length(cutoff))
    Recall <- rep(0, length(cutoff))
    for (i in seq_len(length(cutoff))) {
        idx <- prob_val >= cutoff[i]
        Recall[i] <- mean(idx)
        Precision[i] <- mean((assign_0 == assign_1)[idx])
    }
    if (add_cut1) {
        cutoff <- c(cutoff, 1.0)
        Recall <- c(Recall, 0.0)
        Precision <- c(Precision, 1.0)
    }

    AUC <- 0.0
    for (i in seq_len(length(cutoff) - 1)) {
        AUC <- ((Recall[i] - Recall[i + 1]) *
            (Precision[i] + Precision[i + 1]) * 0.5 + AUC)
    }
    AUC <- AUC / (Recall[1] - Recall[length(Recall)])

    df <- data.frame(
        "cutoff" = cutoff, "Recall" = Recall,
        "Precision" = Precision
    )
    list("df" = df, "AUC" = AUC)
}

#' Scoring the simulation in assignment of singlets and doublets
#' 
#' @param prob Probability matrix for each cell to each component
#' @param I_sim The true identity of assignment from simulation
#' @param cutoff A list of cutoffs from 0 to 1
#' 
#' @return A list with components: df_sg, the recall/precision data.frame 
#' calculated by \code{multiPRC()}, AUC_sg, the AUC calculated by 
#' \code{multiPRC()}, df_db, the recall/precision data.frame calculated by
#' \code{binaryPRC()} and AUC_db the AUC calculated by \code{binaryPRC()}. 
#' Note that \code{multiPRC()} is run on a multiclass version of the problem
#' and \code{binaryPRC} is run on a binarised version of the problem.
#' 
#' @export
#'
assign_scores <- function(prob, I_sim, cutoff = seq(0, 1, 0.001)) {
    col_idx <- colMatch(I_sim, prob)
    print(t(col_idx))

    ass_sg <- multiPRC(prob[, col_idx], I_sim,
        multiLabel.rm = TRUE,
        cutoff = cutoff
    )
    ass_sg$AUC

    ass_db <- binaryPRC(rowSums(prob), rowSums(I_sim > 0) > 1,
        cut_direction = "<=", cutoff = cutoff
    )
    ass_db$AUC

    list(
        "df_sg" = ass_sg$df, "df_db" = ass_db$df,
        "AUC_sg" = ass_sg$AUC, "AUC_db" = ass_db$AUC
    )
}

# Tests for a set of methods
# library(cardelino); library(testthat); source("test-misc.R")

context("test assessment.R")

test_that("mtx_to_df works as expected", {
    df <- mtx_to_df(matrix(seq(12), nrow = 3))
    expect_is(df, "data.frame")
})

test_that("rowMax works as expected", {
    matA <- matrix(sample(seq(12)), nrow = 3)
    rMax <- rowMax(matA)
    expect_is(rMax, "numeric")
})

test_that("rowArgmax works as expected", {
    matA <- matrix(sample(seq(12)), nrow = 3)
    rArg <- rowArgmax(matA)
    expect_is(rArg, "numeric")
})

test_that("colMatch works as expected", {
    matA <- matrix(sample(seq(12)), nrow = 3)
    col_idx <- sample(4)
    matB <- matA[, col_idx]
    cIdx <- colMatch(matB, matA)
    expect_is(cIdx, "numeric")
})


context("test read_data.R")
cell_vcf <- system.file("extdata", "cellSNP.cells.vcf.gz", package = "cardelino")


cell_data <- load_cellSNP_vcf(cell_vcf,
    max_other_allele = NULL,
    min_count = 0, min_MAF = 0
)

test_that("load_cellSNP_vcf works as expected", {
    expect_is(cell_data$A, "dgCMatrix")
    expect_is(cell_data$D, "dgCMatrix")
    expect_identical(dim(cell_data$D), dim(cell_data$A))
})

donor_GT <- load_GT_vcf(cell_vcf, na.rm = FALSE)

test_that("load_GT_vcf works as expected", {
    expect_is(donor_GT$GT, "matrix")
})


context("test tree_utils.R")
data("example_donor")
test_that("get_tree works as expected", {
    tree_obj <- get_tree(tree$Z)
    expect_is(tree_obj, "phylo")
})

# Functions for clone ID

#' Infer clonal identity of single cells
#'
#' @param A variant x cell matrix of integers; number of alternative allele
#' reads in variant i cell j
#' @param D variant x cell matrix of integers; number of total reads covering
#' variant i cell j
#' @param Config variant x clone matrix of binary values. The clone-variant
#' configuration, which encodes the phylogenetic tree structure. This is the
#' output Z of Canopy
#' @param n_clone integer(1), the number of clone to reconstruct. This is in use
#' only if Config is NULL
#' @param relax_Config logical(1), If TRUE, relaxing the Clone Configuration by
#' changing it from fixed value to act as a prior Config with a relax rate.
#' @param relax_rate_fixed numeric(1), If the value is between 0 to 1,
#' the relax rate will be set as a fix value during updating clone Config. If
#' NULL, the relax rate will be learned automatically with relax_rate_prior.
#' @param n_chain integer(1), the number of chains to run, which will be
#' averaged as an output result
#' @param n_proc integer(1), the number of processors to use. This parallel
#' computing can largely reduce time when using multiple chains
#' @param inference character(1), the method to use for inference, either
#' "sampling" to use Gibbs sampling (default) or "EM" to use
#' expectation-maximization (faster)
#' @param verbose logical(1), should the function output verbose information as
#' it runs?
#' @param ... arguments passed to \code{\link{clone_id_Gibbs}} or
#' \code{\link{clone_id_EM}} (as appropriate)
#' @param Psi A vector of float. The fractions of each clone, output P of Canopy
#' @param min_iter A integer. The minimum number of iterations in the Gibbs
#' sampling. The real iteration may be longer until the convergence.
#' @param max_iter A integer. The maximum number of iterations in the Gibbs
#' sampling, even haven't passed the convergence diagnosis
#'
#' @return
#' If inference method is "EM", a list containing \code{theta}, a vector of
#' two floats denoting the parameters of the two components of the base model,
#' i.e., mean of Bernoulli or binomial model given variant exists or not,
#' \code{prob}, the matrix of posterior probabilities of each cell belonging to
#' each clone with fitted parameters, and \code{logLik}, the log likelihood of
#' the final parameters.
#'
#' If inference method is "sampling", a list containing: \code{theta0}, the mean
#' of sampled false positive parameter values; \code{theta1} the mean of sampled
#' (1 - false negative rate) parameter values; \code{theta0_all}, all sampled
#' false positive parameter values; \code{theta1_all}, all sampled (1 - false
#' negative rate) parameter values; \code{element}; \code{logLik_all},
#' log-likelihood for model for all sampled parameter sets; \code{prob_all};
#' \code{prob}, matrix with mean of sampled cell-clone assignment posterior
#' probabilities (the key output of the model); \code{prob_variant}.
#'
#' @details
#' The two Bernoulli components correspond to false positive and false negative
#' rates. The two binomial components correspond to the read distributions
#' with and without the mutation present.
#'
#' @author Yuanhua Huang and Davis McCarthy
#'
#' @import stats
#' @name Clone ID
#' @rdname clone_id
#' @export
#'
#' @examples
#' data(example_donor)
#' assignments <- clone_id(A_clone, D_clone,
#'     Config = tree$Z,
#'     min_iter = 800, max_iter = 1200
#' )
#' prob_heatmap(assignments$prob)
#'
#' assignments_EM <- clone_id(A_clone, D_clone,
#'     Config = tree$Z,
#'     inference = "EM"
#' )
#' prob_heatmap(assignments_EM$prob)
clone_id <- function(A, D, Config = NULL, n_clone = NULL, Psi = NULL,
    relax_Config = TRUE, relax_rate_fixed = NULL,
    inference = "sampling", n_chain = 1, n_proc = 1,
    verbose = TRUE, ...) {
    inference <- match.arg(inference, c("sampling", "EM"))
    ## check input data
    if (!(all(rownames(A) == rownames(D)))) {
        stop("Rownames for A and D are not identical.")
    }
    if (!(all(colnames(A) == colnames(D)))) {
        stop("Colnames for A and D are not identical.")
    }
    if (is.null(Config) && is.null(n_clone)) {
        stop("Config and n_clone can't be NULL together.")
    }
    if (any(A %% 1 != 0, na.rm = TRUE) || 
        any(D %% 1 != 0, na.rm = TRUE)) {
        stop("A and/or D contain non-integer values.")
    }
    ## Cardelino-free mode if Config is NULL
    if (is.null(Config)) {
        cat("Config is NULL: de-novo mode is in use.\n")
        Config <- matrix(0, nrow = nrow(D), ncol = n_clone)
        rownames(Config) <- rownames(D)
        colnames(Config) <- paste0("Clone", seq_len(n_clone))
        relax_Config <- TRUE
        relax_rate_fixed <- 0.5
    }

    ## Match exome-seq and scRNA-seq data
    if (!any(rownames(D) %in% rownames(Config))) {
        stop("No matches in variant names between Config and D arguments.")
    }
    ## match variants
    common_vars <- intersect(rownames(Config), rownames(D))
    A <- A[common_vars, , drop = FALSE]
    D <- D[common_vars, , drop = FALSE]
    Config <- Config[common_vars, , drop = FALSE]
    if (verbose) {
        message(length(common_vars), " variants used for cell assignment.")
    }

    ## change sparse matrix to dense matrix
    A <- as.matrix(A)
    D <- as.matrix(D)

    ## pass data to specific functions
    if (inference == "sampling") {
        # Try to run sampling in parallel. 
        if (n_proc > 1) {
            
            # doMC is not available on Windows so there we fall back to 
            # running the chains using only one processor. 
            if (!requireNamespace("doMC", quietly = TRUE)) {
                message("Parallel sampling is not supported on Windows.")
                message("Falling back to sampling using one processor.")
                ids_list <- list(
                    clone_id_Gibbs(A, D, Config,
                                   Psi = Psi,
                                   relax_Config = relax_Config,
                                   relax_rate_fixed = relax_rate_fixed,
                                   verbose = verbose, ...
                    )
                )     
            } else {
                doMC::registerDoMC(n_proc)
                `%dopar%` <- foreach::`%dopar%`
                
                ids_list <- foreach::foreach(ii = seq_len(n_chain)) %dopar% {
                    clone_id_Gibbs(A, D, Config,
                                   Psi = Psi,
                                   relax_Config = relax_Config,
                                   relax_rate_fixed = relax_rate_fixed,
                                   verbose = verbose, ...
                    )
                }
            }
        } else {
            ids_list <- lapply(seq_len(n_chain), function(x)
                clone_id_Gibbs(A, D, Config,
                    Psi = Psi,
                    relax_Config = relax_Config,
                    relax_rate_fixed = relax_rate_fixed,
                    verbose = verbose, ...
                )
            )
        }
        ids_out <- ids_list[[1]]
        ids_out$n_chain <- 1
        if (n_chain > 1) {
            for (ii in seq(2, n_chain)) {
                ids_out$n_chain <- ids_out$n_chain + 1
                idx <- colMatch(ids_out$prob, ids_list[[ii]]$prob, force = TRUE)
                ids_out$prob <- ids_out$prob + ids_list[[ii]]$prob[, idx]
                ids_out$relax_rate <- ids_out$relax_rate +
                    ids_list[[ii]]$relax_rate
                ids_out$Config_prob <- (ids_out$Config_prob +
                    ids_list[[ii]]$Config_prob[, idx])
            }
            ids_out$prob <- ids_out$prob / n_chain
            ids_out$relax_rate <- ids_out$relax_rate / n_chain
            ids_out$Config_prob <- ids_out$Config_prob / n_chain
        }
        return(ids_out)
    }
    else {
        return(clone_id_EM(A, D, Config, verbose = verbose, ...))
    }
}


#' Assign cells to clones from cardelino results
#'
#' @param prob_mat numeric matrix (cells x clones) of clone posterior
#' probabilities as output by \code{\link{clone_id}}
#' @param threshold numeric(1), posterior probability threshold for cell-clone
#' assignment: if posterior probability is above threshold, assign cell to
#' clone, otherwise leave cell "unassigned"
#'
#' @return a \code{data.frame} with cell ID, assigned clone label and maximum
#' posterior probability across clones.
#'
#' @author Davis McCarthy
#' @export
#' @examples
#' data(example_donor)
#' assignments <- clone_id(A_clone, D_clone, Config = tree$Z, inference = "EM")
#' df <- assign_cells_to_clones(assignments$prob)
#' head(df)
#' table(df$clone)
assign_cells_to_clones <- function(prob_mat, threshold = 0.5) {
    assigns <- data.frame(
        cell = rownames(prob_mat),
        clone = rep("unassigned", nrow(prob_mat)),
        prob_max = rep(NA, nrow(prob_mat)),
        stringsAsFactors = FALSE
    )
    for (i in seq_len(nrow(prob_mat))) {
        assigns[i, "prob_max"] <- max(prob_mat[i, ])
        if (max(prob_mat[i, ]) > threshold) {
            assigns[i, "clone"] <- names(which.max(prob_mat[i, ]))
        }
    }
    assigns
}


#' Assign cells to clones using an EM algorithm
#'
#' @param logLik_threshold A float. The threshold of logLikelihood increase for
#' detecting convergence.
#'
#' @return a list containing \code{theta}, a vector of two floats denoting the
#' binomial rates given variant exists or not, \code{prob}, the matrix of
#' posterior probabilities of each cell belonging to each clone with fitted
#' parameters, and \code{logLik}, the log likelihood of the final parameters.
#'
#' @author Yuanhua Huang
#' @import stats
#' @rdname clone_id
#' @export
#'
clone_id_EM <- function(A, D, Config, Psi = NULL, min_iter = 10,
    max_iter = 1000, logLik_threshold = 1e-5,
    verbose = TRUE) {
    if (is.null(Psi)) {
        Psi <- rep(1 / ncol(Config), ncol(Config))
    }
    if (dim(A)[1] != dim(D)[1] || dim(A)[2] != dim(D)[2] ||
        dim(A)[1] != dim(Config)[1] || dim(Config)[2] != length(Psi)) {
        stop(
            "A and D must have the same size;\n",
            "A and Config must have the same number of variants;\n",
            "Config and Psi must have the same number of clones"
        )
    }

    ## preprocessing
    N <- dim(A)[1] # number of variants
    M <- dim(A)[2] # number of cells
    K <- dim(Config)[2] # number of clones
    A[which(D == 0)] <- NA
    D[which(D == 0)] <- NA
    A[(D > 0) & is.na(A)] <- 0

    C1 <- Config
    C0 <- 1 - Config
    A1 <- A # number of alteration reads
    B1 <- D - A # number of reference reads
    W_log <- sum(lchoose(D, A), na.rm = TRUE) # log binomial coefficients

    A1[is.na(A1)] <- 0
    B1[is.na(B1)] <- 0

    S1 <- t(A1) %*% C0 # Alteration reads without variants (C=0): theta[1]
    S2 <- t(B1) %*% C0 # Reference reads without variants  (C=0): 1-theta[1]
    S3 <- t(A1) %*% C1 # Alteration reads with variants (C=1): theta[2]
    S4 <- t(B1) %*% C1 # Reference reads with variants  (C=1): 1-theta[2]

    ## random initialization for EM
    theta <- c(stats::runif(1, 0.001, 0.25), stats::runif(1, 0.3, 0.6))
    logLik <- 0
    logLik_new <- logLik + logLik_threshold * 2

    ## EM iterations
    for (it in seq_len(max_iter)) {
        # Check convergence
        if ((it > min_iter) && ((logLik_new - logLik) < logLik_threshold)) {
            break
        }
        logLik <- logLik_new

        # E-step
        logLik_mat <- (S1 * log(theta[1]) + S2 * log(1 - theta[1]) +
            S3 * log(theta[2]) + S4 * log(1 - theta[2]))
        logLik_mat <- t(t(logLik_mat) + log(Psi))

        logLik_vec <- rep(NA, nrow(logLik_mat))
        for (i in seq_len(nrow(logLik_mat))) {
            logLik_vec[i] <- matrixStats::logSumExp(logLik_mat[i, ],
                na.rm = TRUE
            )
        }
        logLik_new <- sum(logLik_vec, na.rm = TRUE) + W_log
        logLik_mat_amplify <- logLik_mat - matrixStats::rowMaxs(logLik_mat)
        prob_mat <- exp(logLik_mat_amplify) / rowSums(exp(logLik_mat_amplify))

        # M-step
        if (is.na(sum(prob_mat * (S1 + S2))) ||
            sum(prob_mat * (S1 + S2)) == 0) {
            theta[1] <- 0.02
        } else {
            theta[1] <- sum(prob_mat * S1) / sum(prob_mat * (S1 + S2))
        }
        if (is.na(sum(prob_mat * (S3 + S4))) ||
            sum(prob_mat * (S3 + S4)) == 0) {
            theta[2] <- 0.75
        } else {
            theta[2] <- sum(prob_mat * S3) / sum(prob_mat * (S3 + S4))
        }
    }
    if (verbose) {
        print(paste("Total iterations:", it))
    }

    ## return values
    return_list <- list("theta" = theta, "prob" = prob_mat, "logLik" = logLik)
    return_list
}


#' Assign cells to clones using a Gibbs sampling algorithm
#'
#' @param prior0 numeric(2), alpha and beta parameters for the Beta prior
#' distribution on the inferred false positive rate.
#' @param prior1 numeric(2), alpha and beta parameters for the Beta prior
#' distribution on the inferred (1 - false negative) rate.
#' @param relax_rate_prior numeric(2), the two parameters of beta prior
#' distribution of the relax rate for relaxing the clone Configuration. This
#' mode is used when relax_relax is NULL.
#' @param keep_base_clone bool(1), if TRUE, keep the base clone of Config to its
#' input values when relax mode is used.
#' @param buin_frac numeric(1), the fraction of chain as burn-in period
#' @param wise A string, the wise of parameters for theta1: global, variant,
#' element.
#' @param relabel bool(1), if TRUE, relabel the samples of both Config and prob
#' during the Gibbs sampling.
#'
#' @author Yuanhua Huang
#' @import matrixStats
#' @rdname clone_id
#' @export
#'
clone_id_Gibbs <- function(A, D, Config, Psi = NULL,
    relax_Config = TRUE, relax_rate_fixed = NULL,
    relax_rate_prior = c(1, 9), keep_base_clone = TRUE,
    prior0 = c(0.2, 99.8), prior1 = c(0.45, 0.55),
    min_iter = 5000, max_iter = 20000, buin_frac = 0.5,
    wise = "variant", relabel = FALSE, verbose = TRUE) {
    if (is.null(Psi)) {
        Psi <- rep(1 / ncol(Config), ncol(Config))
    }
    if (dim(A)[1] != dim(D)[1] || dim(A)[2] != dim(D)[2] ||
        dim(A)[1] != dim(Config)[1] || dim(Config)[2] != length(Psi)) {
        stop(
            "A and D must have the same size;\n ",
            "A and Config must have the same number of variants;\n",
            "Config and Psi must have the same number of clones"
        )
    }
    if (sum(c("element", "variant", "global") == wise) == 0) {
        stop(
            "Input wise mode: ", wise,
            ", while only supporting: element, variant, global"
        )
    }

    ## preprocessing
    N <- dim(A)[1] # number of variants
    M <- dim(A)[2] # number of cells
    K <- dim(Config)[2] # number of clones

    A[which(D == 0)] <- NA
    D[which(D == 0)] <- NA
    A[(D > 0) & is.na(A)] <- 0

    C1 <- Config
    C0 <- 1 - Config
    A1 <- A # number of alteration reads
    B1 <- D - A # number of reference reads
    W_log <- sum(lchoose(D, A), na.rm = TRUE) # log binomial coefficients

    A1[is.na(A1)] <- 0
    B1[is.na(B1)] <- 0

    # reads number list for each clone
    S1_list <- list()
    S2_list <- list()
    S3_list <- list()
    S4_list <- list()
    for (k in seq_len(K)) {
        S1_list[[k]] <- A1 * C0[, k]
        S2_list[[k]] <- B1 * C0[, k]
        S3_list[[k]] <- A1 * C1[, k]
        S4_list[[k]] <- B1 * C1[, k]
    }

    ## Prepare for sampling
    if (wise == "global") {
        idx_vec <- seq_len(1) # For: theta1_all[t, ] <- theta1[idx_vec]
        idx_mat <- seq_len(N * M) # For: theta1[idx_mat] <- theta1_all[t, ]
    } else if (wise == "variant") {
        idx_vec <- seq_len(N)
        idx_mat <- seq_len(N * M)
    } else if (wise == "element") {
        idx_vec <- which(A1 + B1 > 0)
        idx_mat <- which(A1 + B1 > 0)
    }
    n_element <- length(idx_vec)

    if (is.null(dim(prior1)) && length(prior1) == 2) {
        # two variable to a matrix
        prior1 <- t(matrix(rep(prior1, n_element), nrow = 2))
    }
    if (!is.matrix(prior1)) {
        stop("prior1 need to be a matrix of n_element x 2")
    }

    prob_all <- matrix(0, nrow = max_iter, ncol = M * K)
    logLik_mat <- matrix(0, nrow = M, ncol = K)
    logLik_all <- matrix(0, nrow = max_iter, ncol = 1)
    assign_all <- matrix(0, nrow = max_iter, ncol = M)
    theta0_all <- matrix(0, nrow = max_iter, ncol = 1)
    theta1_all <- matrix(0, nrow = max_iter, ncol = n_element)
    Config_all <- matrix(0, nrow = max_iter, ncol = N * K)
    relax_rate_all <- matrix(0, nrow = max_iter, ncol = 1)

    if (!is.null(relax_Config) && relax_Config != FALSE) {
        if (!is.null(relax_rate_fixed)) {
            if (relax_rate_fixed > 1 || relax_rate_fixed < 0) {
                stop("relax_rate_fixed needs to be NULL or in [0, 1].")
            }
            relax_rate <- relax_rate_fixed ## fixed relax_rate
            relax_rate_all[, ] <- relax_rate
        } else if (!is.null(relax_rate_prior)) {
            relax_rate <- relax_rate_prior[1] / (relax_rate_prior[1] +
                relax_rate_prior[2])
        } else {
            stop("Require value for either relax_Config or relax_prior.")
        }

        Config_new <- Config
        Config_prior <- Config
        Config_prior[Config == 1] <- 1 - relax_rate
        Config_prior[Config == 0] <- relax_rate
        if (keep_base_clone) {
            Config_prior[, 1] <- Config[, 1]
        }
        Config_prior_oddlog <- log(Config_prior) - log(1 - Config_prior)
        Iden_mat <- matrix(0, nrow = M, ncol = K)
    }

    ## Random initialization
    theta0_all[1, 1] <- stats::rbeta(1, prior0[1], prior0[2])
    theta1_all[1, ] <- stats::rbeta(rep(1, n_element), prior1[, 1], prior1[, 2])
    theta1 <- matrix(NA, nrow = N, ncol = M)

    ## Gibbs sampling
    for (it in 2:max_iter) {
        # Update prob_mat
        theta0 <- theta0_all[it - 1, 1]
        theta1[idx_mat] <- theta1_all[it - 1, ]
        for (k in seq_len(K)) {
            logLik_mat[, k] <- (colSums(S1_list[[k]] * log(theta0),
                na.rm = TRUE
            ) +
                colSums(S2_list[[k]] * log(1 - theta0), na.rm = TRUE) +
                colSums(S3_list[[k]] * log(theta1), na.rm = TRUE) +
                colSums(S4_list[[k]] * log(1 - theta1), na.rm = TRUE))
            logLik_mat[, k] <- logLik_mat[, k] + log(Psi[k])
        }
        logLik_mat_amplify <- logLik_mat - matrixStats::rowMaxs(logLik_mat)
        prob_mat <- exp(logLik_mat_amplify) / rowSums(exp(logLik_mat_amplify))
        prob_all[it, ] <- prob_mat

        # logLik_vec <- rep(NA, nrow(logLik_mat))
        # for (i in seq_len(nrow(logLik_mat))) {
        #     logLik_vec[i] <- matrixStats::logSumExp(logLik_mat[i,],
        #                                             na.rm = TRUE)
        # }
        # logLik_all[it] <- sum(logLik_vec, na.rm = TRUE) + W_log
        # #Update logLikelihood (TODO: add W0 and W1)

        # Sample assignment
        for (j in seq_len(M)) {
            assign_all[it, j] <- sample(seq_len(K), 1,
                replace = TRUE,
                prob = prob_mat[j, ]
            )
        }

        ## Update Config
        if (!is.null(relax_Config) && relax_Config != FALSE) {
            if (it > (0.1 * min_iter + 5) && is.null(relax_rate_fixed)) {
                diff0 <- sum((Config == Config_new)[, 2:ncol(Config)])
                diff1 <- sum((Config != Config_new)[, 2:ncol(Config)])
                relax_rate <- stats::rbeta(
                    1, relax_rate_prior[1] + diff1,
                    relax_rate_prior[2] + diff0
                )
                relax_rate_all[it] <- relax_rate

                Config_prior <- Config
                Config_prior[Config == 1] <- 1 - relax_rate
                Config_prior[Config == 0] <- relax_rate
                if (keep_base_clone) {
                    Config_prior[, 1] <- Config[, 1]
                }
                Config_prior_oddlog <- log(Config_prior) - log(1 - Config_prior)
            }

            Iden_mat[, ] <- 0
            for (j in seq_len(M)) {
                Iden_mat[j, assign_all[it, j]] <- 1
            }

            # calculate log_probability matrix with genotype 0 and 1
            P0_mat <- A1 * log(theta0) + B1 * log(1 - theta0) + W_log
            P1_mat <- A1 * log(theta1) + B1 * log(1 - theta1) + W_log

            oddR_log <- P1_mat %*% Iden_mat - P0_mat %*% Iden_mat
            oddR_log <- oddR_log + Config_prior_oddlog
            oddR_log[which(oddR_log > 50)] <- 50
            oddR_log[which(oddR_log < -50)] <- -50
            Config_prob_tmp <- exp(oddR_log) / (exp(oddR_log) + 1)

            Config_new[, ] <- stats::rbinom(N * K, size = 1, Config_prob_tmp)
            Config_all[it, ] <- Config_new

            for (k in seq_len(K)) {
                S1_list[[k]] <- A1 * (1 - Config_new[, k])
                S2_list[[k]] <- B1 * (1 - Config_new[, k])
                S3_list[[k]] <- A1 * Config_new[, k]
                S4_list[[k]] <- B1 * Config_new[, k]
            }
        }

        # Sample theta with assigned clones
        S1_wgt <- S2_wgt <- 0 # weighted S1
        S3_wgt <- S4_wgt <- matrix(0, nrow = N, ncol = M)
        for (k in seq_len(K)) {
            idx <- which(assign_all[it, ] == k)
            S1_wgt <- S1_wgt + sum(S1_list[[k]][, idx], na.rm = TRUE)
            S2_wgt <- S2_wgt + sum(S2_list[[k]][, idx], na.rm = TRUE)
            S3_wgt[, idx] <- S3_wgt[, idx] + S3_list[[k]][, idx]
            S4_wgt[, idx] <- S4_wgt[, idx] + S4_list[[k]][, idx]
        }

        if (wise == "global") {
            S3_wgt[, ] <- sum(S3_wgt, na.rm = TRUE)
            S4_wgt[, ] <- sum(S4_wgt, na.rm = TRUE)
        } else if (wise == "variant") {
            S3_wgt[, ] <- rowSums(S3_wgt, na.rm = TRUE)
            S4_wgt[, ] <- rowSums(S4_wgt, na.rm = TRUE)
        }
        theta0_all[it, 1] <- stats::rbeta(
            1, prior0[1] + S1_wgt,
            prior0[2] + S2_wgt
        )
        theta1_all[it, ] <- stats::rbeta(
            rep(1, n_element),
            prior1[, 1] + S3_wgt[idx_vec],
            prior1[, 2] + S4_wgt[idx_vec]
        )

        # Calculate logLikelihood
        logLik_all[it] <- get_logLik(
            A1, B1, Config_new, assign_all[it, ],
            theta0, theta1
        )

        # Check convergence. TODO: consider relabel
        if ((it >= min_iter) && (it %% 100 == 0)) {
            Converged_all <- abs(Geweke_Z(prob_all[seq_len(it), ])) <= 2
            # Converged_all <- abs(Geweke_Z(Config_all[1:it, ])) <= 2
            if (verbose) {
                cat(paste0(
                    round(mean(Converged_all, na.rm = TRUE), 3) * 100,
                    "% converged.\n"
                ))
            }
            if (mean(Converged_all, na.rm = TRUE) > 0.995) {
                break
            }
        }
    }
    Converged_all <- abs(Geweke_Z(prob_all[seq_len(it), ])) <= 2
    percent_converged <- round(mean(Converged_all, na.rm = TRUE), 3) * 100
    if (percent_converged <= 95) {
        warning("Only ", percent_converged, "% of parameters converged. ", 
                "Consider increasing `max_iter`.", call. = FALSE)
    } else {
        print(paste("Converged in", it, "iterations.")) 
    }

    ## Return values
    n_buin <- ceiling(it * buin_frac)

    a <- A1[idx_mat]
    d <- A1[idx_mat] + B1[idx_mat]
    binom_pdf1 <- binom_pdf0 <- rep(0, n_element)
    for (i in seq(n_buin, it)) {
        binom_pdf1 <- binom_pdf1 + stats::dbinom(a,
            size = d,
            prob = theta1_all[i, ]
        )
        binom_pdf0 <- binom_pdf0 + stats::dbinom(a,
            size = d,
            prob = theta0_all[i]
        )
    }
    prob_variant <- matrix(NA, nrow = N, ncol = M)
    prob_variant[idx_mat] <- binom_pdf1 / (binom_pdf1 + binom_pdf0)
    row.names(prob_variant) <- row.names(A)
    colnames(prob_variant) <- colnames(A)

    if (relabel) {
        col_idx_use <- seq(K)
        for (ii in seq(n_buin, it)) {
            mat1 <- matrix(prob_all[ii - 1, ], nrow = M)
            mat2 <- matrix(prob_all[ii, ], nrow = M)
            # mat1 <- matrix(Config_all[ii - 1, ], nrow = N)
            # mat2 <- matrix(Config_all[ii, ], nrow = N)

            if (ncol(mat1) <= 5) {
                idx <- colMatch(mat1, mat2, force = TRUE)
            }
            else {
                idx <- colMatch(mat1, mat2, force = FALSE)
            }
            col_idx_use <- col_idx_use[idx]
            prob_all[ii, ] <- matrix(prob_all[ii, ], nrow = M)[, col_idx_use]
            Config_all[ii, ] <- matrix(Config_all[ii, ],
                nrow = N
            )[, col_idx_use]
        }
    }
    prob_mat <- matrix(colMeans(prob_all[n_buin:it, ]), nrow = M)
    row.names(prob_mat) <- colnames(A)
    colnames(prob_mat) <- colnames(Config)

    Config_prob <- Config
    Config_prob[, ] <- colMeans(Config_all[n_buin:it, ])

    theta0 <- mean(theta0_all[n_buin:it, ])
    theta1[idx_mat] <- colMeans(as.matrix(theta1_all[n_buin:it, ]))


    logLik_post <- get_logLik(A1, B1, Config_prob, prob_mat, theta0, theta1)
    DIC <- devianceIC(logLik_all[n_buin:it], logLik_post)

    #     DIC <- devianceIC(logLik_all[n_buin:it],
    #                       A1, B1, Config_prob, theta0, theta1, Psi, W_log)

    return_list <- list(
        "theta0" = theta0, "theta1" = theta1,
        "theta0_all" = as.matrix(theta0_all[seq_len(it), ]),
        "theta1_all" = as.matrix(theta1_all[seq_len(it), ]),
        "element" = idx_mat, "logLik" = logLik_all[seq_len(it)],
        "prob_all" = prob_all[seq_len(it), ],
        "prob" = prob_mat, "prob_variant" = prob_variant,
        "relax_rate" = mean(relax_rate_all[n_buin:it]),
        "Config_prob" = Config_prob,
        "Config_all" = Config_all[seq_len(it), ],
        "relax_rate_all" = relax_rate_all[seq_len(it)], "DIC" = DIC
    )
    return_list
}


#' Geweke diagnostic for MCMC sampling.
#'
#' @param X A matrix of MCMC samples for N samples per K variables
#' @param first A float between 0 and 1. The initial region of MCMC chain.
#' @param last A float between 0 and 1. The final region of MCMC chain.
#'
#' @author Yuanhua Huang
#' @return \code{Z}, a vector of absolute value of Z scores for each variable.
#' When |Z| <= 2, the sampling could be taken as converged.
#'
Geweke_Z <- function(X, first = 0.1, last = 0.5) {
    # The original Geweke’s diagnostics uses pectral densities to estimate
    # the sample variances (Geweke,1992), for adjusting the samples
    # as the two ‘samples’ are not independent
    # so as the implementation in coda::geweke.diag()

    # Converged_all = rep(NA, ncol(prob_all))
    # for (ic in seq_len(length(Converged_all))) {
    # Converged_all[ic] = abs(coda::geweke.diag(prob_all[, ic])$z) <= 2
    # }

    if (is.null(dim(X))) {
        X <- as.matrix(X, ncol = 1)
    }
    N <- nrow(X)
    A <- X[seq_len(floor(first * N)), , drop = FALSE]
    B <- X[ceiling(last * N):N, , drop = FALSE]

    A_col_mean <- colMeans(A)
    B_col_mean <- colMeans(B)
    A_col_var <- rowSums((t(A) - A_col_mean)^2) / (nrow(A) - 1) # ^2
    B_col_var <- rowSums((t(B) - B_col_mean)^2) / (nrow(A) - 1) # ^2

    min_var <- 10^(-50)
    Z <- (A_col_mean - B_col_mean) / sqrt(A_col_var + B_col_var + min_var)

    Z
}

#' Log likelihood of clone_id model
#' It returns P(A, D | C, I, theta0, theta1)
#'
#' @param A1 variant x cell matrix of integers; number of alternative allele
#' reads in variant i cell j
#' @param B1 variant x cell matrix of integers; number of reference allele
#' reads in variant i cell j
#' @param Config variant x clone matrix of float values. The clone-variant
#' configuration probability, averaged by posterior samples
#' @param Assign cells x clone matrix of float values. The cell-clone
#' assignment probability, averaged by posterior samples
#' @param theta0 the binomial rate for alternative allele from config = 0
#' @param theta1 the binomial rate for alternative allele from config = 1
#'
#' @author Yuanhua Huang
#' @return \code{logLik}, a float of log likelihood
#'
get_logLik <- function(A1, B1, Config, Assign, theta0, theta1) {
    if (is.null(dim(Assign)) || length(dim(Assign)) == 1) {
        Assign_prob <- matrix(0, length(Assign), ncol(Config))
        for (i in seq_len(length(Assign))) {
            Assign_prob[i, Assign[i]] <- 1
        }
    } else {
        Assign_prob <- Assign
    }

    prob_mat <- Config %*% t(Assign_prob)

    Lik_mat <- (exp(log(theta1) * A1 + log(1 - theta1) * B1) * prob_mat +
        exp(log(theta0) * A1 + log(1 - theta0) * B1) * (1 - prob_mat))

    logLik <- (sum(log(Lik_mat), na.rm = TRUE) +
        sum(lchoose(A1 + B1, A1), na.rm = TRUE))
    logLik
}


#' Deviance Information Criterion for cardelino model
#'
#' @param logLik_all A vector of numeric; the log likelihood of posterior
#' sample, i.e., posterior samples of deviance
#' @param logLik_post numeric(1); the log likelihood of mean posterior
#' parameters, i.e., deviance of posterior means
#'
#' @author Yuanhua Huang
#' @return \code{DIC}, a float of deviance information criterion
#'
devianceIC <- function(logLik_all, logLik_post) {
    # https://www.mrc-bsu.cam.ac.uk/wp-content/uploads/DIC-slides.pdf
    # Spiegelhalter et al. The deviance information criterion: 12 years on, 2014
    # Spiegelhalter et al. Bayesian measures of model complexity and fit, 2002
    # Gelman et al. Bayesian Data Analysis. 3rd Edition, 2013
    # (Charpter 7.2, p173)

    logLik_mean <- mean(logLik_all)
    logLik_var <- var(logLik_all)

    p_D_Spiegelhalter <- -2 * logLik_mean - (-2 * logLik_post)
    DIC_Spiegelhalter <- -2 * logLik_post + 2 * p_D_Spiegelhalter

    p_D_Gelman <- 2 * logLik_var
    DIC_Gelman <- -2 * logLik_post + 2 * p_D_Gelman

    DIC <- DIC_Gelman

    cat(paste(
        "DIC:", round(DIC, 2),
        "D_mean:", round(-2 * logLik_mean, 2),
        "D_post:", round(-2 * logLik_post, 2),
        "logLik_var:", round(logLik_var, 2), "\n"
    ))

    list(
        "DIC" = DIC,
        "logLik_var" = logLik_var,
        "D_mean" = -2 * logLik_mean,
        "D_post" = -2 * logLik_post,
        "DIC_Gelman" = DIC_Gelman,
        "DIC_Spiegelhalter" = DIC_Spiegelhalter
    )
}

#' Read a VCF file into R session
#'
#' @param vcf_file character(1), path to VCF file to read into R session as a
#' \code{\link[VariantAnnotation]{CollapsedVCF}} object
#' @param genome character(1), string indicating the genome build used in the
#' VCF file(s) (default: "GRCh37")
#' @param seq_levels_style character(1), string passed to
#' \code{\link[GenomeInfoDb]{seqlevelsStyle}} the style to use for
#' chromosome/contig names (default: "Ensembl")
#' @param verbose logical(1), should messages be printed as function runs?
#'
#' @return a vcf object
#' @examples
#' vcf <- read_vcf(system.file("extdata", "cells.donorid.vcf.gz",
#'                package = "cardelino"))
#' 
#' @importFrom VariantAnnotation readVcf isSNV geno ref alt
#'  genotypeToSnpMatrix
#' @importFrom GenomeInfoDb seqlevelsStyle seqlengths
#' @importFrom methods as is
#' 
#' @export
#'
read_vcf <- function(vcf_file, genome = "GRCh37",
    seq_levels_style = "Ensembl", verbose = TRUE) {
    ## Read in VCF from this sample
    if (verbose) {
        message("Reading VCF.")
    }
    vcf_sample <- VariantAnnotation::readVcf(vcf_file, genome)
    vcf_sample <- vcf_sample[VariantAnnotation::isSNV(vcf_sample)]
    if (length(vcf_sample) > 0) {
        GenomeInfoDb::seqlevelsStyle(vcf_sample) <- seq_levels_style
        new_snp_names <- gsub(
            "chr", "",
            gsub(
                ":", "_",
                gsub(
                    "_[ATCG]/[ATCG]", "",
                    names(vcf_sample)
                )
            )
        )
        names(vcf_sample) <- new_snp_names
    } else {
        stop("No SNVs present in VCF file.")
    }
    if (verbose) {
        message(
            "Read ", dim(vcf_sample)[1], " variants for ",
            dim(vcf_sample)[2], " samples."
        )
    }
    vcf_sample
}

#' Get SNP data matrices from VCF object(s)
#'
#' @param vcf_cell a \code{\link[VariantAnnotation]{CollapsedVCF}} object
#' containing variant data for cells
#' @param vcf_donor an optional \code{\link[VariantAnnotation]{CollapsedVCF}}
#' object containing genotype data for donors
#' @param verbose logical(1), should the function output verbose information as
#' it runs?
#' @param donors optional character vector providing a set of donors to use, by
#' subsetting the donors present in the \code{donor_vcf_file}; if \code{NULL}
#' (default) then all donors present in VCF will be used.
#'
#' @return a list containing
#' \code{A}, a matrix of integers. Number of alteration reads in SNP i cell j.
#' \code{D}, a matrix of integers. Number of reads depth in SNP i cell j.
#' \code{R}, a matrix of integers. Number of reference reads in SNP i cell j.
#' \code{GT_cells}, a matrix of integers for genotypes. The cell-SNP
#' configuration.
#' \code{GT_donors}, a matrix of integers for genotypes. The donor-SNP
#' configuration.
#' 
#' @examples
#' vcf_cell <- read_vcf(system.file("extdata", "cells.donorid.vcf.gz",
#'                      package = "cardelino"))
#' vcf_donor <-  read_vcf(system.file("extdata", "donors.donorid.vcf.gz",
#'                        package = "cardelino"))
#' snp_data <- get_snp_matrices(vcf_cell, vcf_donor)
#' 
#' @import snpStats
#' @importFrom GenomicRanges findOverlaps
#' @importFrom S4Vectors queryHits subjectHits
#' 
#' @export
#'
get_snp_matrices <- function(vcf_cell, vcf_donor = NULL, verbose = TRUE,
    donors = NULL) {
    if (!methods::is(vcf_cell, "CollapsedVCF")) {
        stop(
            "vcf_cell must be a CollapsedVCF object from the ",
            "VariantAnnotation package."
        )
    }
    vcf_cell <- GenomeInfoDb::sortSeqlevels(vcf_cell, X.is.sexchrom = TRUE)
    slengths_sample <- GenomeInfoDb::seqlengths(vcf_cell)
    if (!is.null(vcf_donor)) {
        if (!methods::is(vcf_donor, "CollapsedVCF")) {
            stop(
                "vcf_donor must be a CollapsedVCF object from the ",
                "VariantAnnotation package."
            )
        }
        ## filter sample VCF to those variants found in donor VCF
        if (!is.null(donors)) {
            if (sum(colnames(vcf_donor) %in% donors) < 1) {
                stop(
                    "No donors in vcf_donor are found in the donors",
                    "argument supplied."
                )
            }
            vcf_donor <- vcf_donor[, colnames(vcf_donor) %in% donors]
        }
        vcf_donor <- GenomeInfoDb::sortSeqlevels(vcf_donor,
            X.is.sexchrom = TRUE
        )
        GenomeInfoDb::seqlengths(vcf_donor) <-
            slengths_sample[GenomeInfoDb::seqlevels(vcf_donor)]
        ovlap <- GenomicRanges::findOverlaps(vcf_cell, vcf_donor)
        if (length(ovlap) < 1L) {
            stop("No variants overlapping in cell VCF and Donor VCF.")
        } else {
            if (verbose) {
                message("Filtering cell VCF.")
            }
            vcf_cell <- vcf_cell[S4Vectors::queryHits(ovlap)]
            if (verbose) {
                message("Filtering donor VCF.")
            }
            vcf_donor <- vcf_donor[S4Vectors::subjectHits(ovlap)]
            match_alleles <- unlist(
                VariantAnnotation::ref(vcf_cell) == 
                VariantAnnotation::ref(vcf_donor) &
                VariantAnnotation::alt(vcf_cell) == 
                VariantAnnotation::alt(vcf_donor)
            )
            if (sum(match_alleles) < 1L) {
                stop(
                    "No variants with matching alleles in sample and ",
                    "donor VCFs"
                )
            }
            vcf_cell <- vcf_cell[match_alleles]
            vcf_donor <- vcf_donor[match_alleles]
            if (verbose) {
                message("Extracting donor SNP genotype matrix.")
            }
            sm_donor <- VariantAnnotation::genotypeToSnpMatrix(
                VariantAnnotation::geno(vcf_donor, "GT"),
                ref = VariantAnnotation::ref(vcf_donor),
                alt = VariantAnnotation::alt(vcf_donor)
            )
        }
    } else {
        sm_donor <- NULL
    }
    ## get snp matrices
    if (verbose) {
        message("Extracting cell SNP matrices.")
    }
    sm_sample <- VariantAnnotation::genotypeToSnpMatrix(
        VariantAnnotation::geno(vcf_cell, "GT"),
        ref = VariantAnnotation::ref(vcf_cell),
        alt = VariantAnnotation::alt(vcf_cell)
    )
    
    sm_sample_REF <- matrix(vapply(
        VariantAnnotation::geno(vcf_cell, "AD"),
        function(x) x[[1]],
        numeric(1)
    ), ncol = ncol(vcf_cell))
    
    sm_sample_ALT <- matrix(vapply(
        VariantAnnotation::geno(vcf_cell, "AD"),
        function(x) x[[2]], 
        numeric(1)    
    ), ncol = ncol(vcf_cell))
    
    sm_sample_ALT[is.na(sm_sample_ALT) & !is.na(sm_sample_REF)] <- 0
    sm_sample_DEP <- sm_sample_REF + sm_sample_ALT
    sm_sample_DEP[sm_sample_DEP == 0] <- NA
    sm_sample_REF[is.na(sm_sample_DEP)] <- NA
    sm_sample_ALT[is.na(sm_sample_DEP)] <- NA
    rownames(sm_sample_REF) <- rownames(vcf_cell)
    rownames(sm_sample_ALT) <- rownames(vcf_cell)
    rownames(sm_sample_DEP) <- rownames(vcf_cell)
    colnames(sm_sample_REF) <- colnames(vcf_cell)
    colnames(sm_sample_ALT) <- colnames(vcf_cell)
    colnames(sm_sample_DEP) <- colnames(vcf_cell)
    na_sample <- is.na(sm_sample_DEP)
    if (sum(!na_sample) < 1L) {
        stop("No variants with non-missing genotypes cells VCF and Donor VCF\n")
    }
    out <- list(
        A = sm_sample_ALT, D = sm_sample_DEP, R = sm_sample_REF,
        GT_cells = t(methods::as(sm_sample$genotypes, "numeric"))
    )
    if (!is.null(vcf_donor)) {
        out[["GT_donors"]] <- t(methods::as(sm_donor$genotypes, "numeric"))
    } else {
        out[["GT_donors"]] <- NULL
    }
    out
}


#' Load sparse matrices A and D from cellSNP VCF file with filtering SNPs
#'
#' @param vcf_file character(1), path to VCF file generated from cellSNP
#' @param max_other_allele maximum ratio of other alleles comparing to REF and
#' ALT alleles; for cellSNP vcf, we recommend 0.05
#' @param min_count minimum count across all cells, e.g., 20
#' @param min_MAF minimum minor allele fraction, e.g., 0.1
#' @param rowname_format the format of rowname: NULL is the default from vcfR,
#' short is CHROM_POS, and full is CHROM_POS_REF_ALT
#' @param keep_GL logical(1), if TRUE, check if GL (genotype probability) exists
#' it will be returned
#' 
#' @return A list with elements the matrices A and D and GL, the genotype 
#'   probability. If keep_GL is false the GL element will be an empty list.
#' 
#' @export
#'
#' @examples
#' vcf_file <- system.file("extdata", "cellSNP.cells.vcf.gz",
#'     package = "cardelino"
#' )
#' input_data <- load_cellSNP_vcf(vcf_file)
load_cellSNP_vcf <- function(vcf_file, min_count = 0, min_MAF = 0,
    max_other_allele = NULL, rowname_format = "full",
    keep_GL = FALSE) {
    vcf_temp <- vcfR::read.vcfR(vcf_file)
    dp_full <- vcfR::extract.gt(vcf_temp, element = "DP", as.numeric = TRUE)
    ad_full <- vcfR::extract.gt(vcf_temp, element = "AD", as.numeric = TRUE)

    idx <- (rowSums(dp_full, na.rm = TRUE) >= min_count &
        ((rowSums(ad_full, na.rm = TRUE) /
            rowSums(dp_full, na.rm = TRUE)) >= min_MAF))

    idx <- idx & (!is.na(idx))

    if (!is.null(max_other_allele)) {
        dp_sum <- vcfR::extract.info(vcf_temp,
            element = "DP",
            as.numeric = TRUE
        )
        oth_sum <- vcfR::extract.info(vcf_temp,
            element = "OTH",
            as.numeric = TRUE
        )
        idx <- idx & (oth_sum / dp_sum < max_other_allele)
    }

    print(paste(sum(idx), "out of", length(idx), "SNPs passed."))

    A <- ad_full[idx, ]
    D <- dp_full[idx, ]
    A[is.na(A)] <- 0
    D[is.na(D)] <- 0

    if (!is.null(rowname_format)) {
        fix_val <- vcfR::getFIX(vcf_temp)
        if (rowname_format == "short") {
            var_ids <- paste0(fix_val[, 1], "_", fix_val[, 2])
        } else {
            var_ids <- paste0(
                fix_val[, 1], "_", fix_val[, 2],
                "_", fix_val[, 4], "_", fix_val[, 5]
            )
        }
    }
    row.names(A) <- row.names(D) <- var_ids[idx]

    GL_list <- list()
    if (keep_GL && sum(strsplit(vcf_temp@gt[1, 1], ":")$FORMAT == "GL") > 0) {
        GL_full <- vcfR::extract.gt(vcf_temp,
            element = "GL",
            as.numeric = FALSE
        )
        for (ii in seq_len(length(strsplit(GL_full[1, 1], ",")[[1]]))) {
            GL_tmp <- vcfR::masplit(GL_full, delim = ",", record = ii, sort = 0)
            GL_tmp[is.na(GL_tmp)] <- 0
            row.names(GL_tmp) <- var_ids
            GL_list[[ii]] <- Matrix::Matrix(GL_tmp[idx, ], sparse = TRUE)
        }
    }

    list(
        "A" = Matrix::Matrix(A, sparse = TRUE),
        "D" = Matrix::Matrix(D, sparse = TRUE),
        "GL" = GL_list
    )
}


#' Load genotype VCF into numeric values: 0, 1, or 2
#'
#' Note, the genotype VCF can be very big for whole genome. It would be more
#' efficient to only keep the wanted variants and samples. bcftools does such
#' jobs nicely.
#'
#' @param vcf_file character(1), path to VCF file for donor genotypes
#' @param rowname_format the format of rowname: NULL is the default from vcfR,
#' short is CHROM_POS, and full is CHROM_POS_REF_ALT
#' @param na.rm logical(1), if TRUE, remove the variants with NA values
#' @param keep_GP logical(1), if TRUE, check if GP (genotype probability) exists
#' it will be returned
#' 
#' @return A list representing the loaded genotype information with two
#'   components: GT, the usual numeric representation of genotype and GP the
#'   genotype probabilities. Note that if \code{keep_GP} is false the GP
#'   component will be NULL.
#' 
#' @export
#'
#' @examples
#' vcf_file <- system.file("extdata", "cellSNP.cells.vcf.gz",
#'     package = "cardelino"
#' )
#' GT_dat <- load_GT_vcf(vcf_file, na.rm = FALSE)
load_GT_vcf <- function(vcf_file, rowname_format = "full", na.rm = TRUE,
    keep_GP = TRUE) {
    GT_vcf <- vcfR::read.vcfR(vcf_file)

    ## variant ids
    if (!is.null(rowname_format)) {
        fix_val <- vcfR::getFIX(GT_vcf)
        if (rowname_format == "short") {
            var_ids <- paste0(fix_val[, 1], "_", fix_val[, 2])
        } else {
            var_ids <- paste0(
                fix_val[, 1], "_", fix_val[, 2],
                "_", fix_val[, 4], "_", fix_val[, 5]
            )
        }
    }

    ## GT values
    GT <- vcfR::extract.gt(GT_vcf, element = "GT")

    GT_num <- matrix(NA, nrow = nrow(GT), ncol = ncol(GT))
    colnames(GT_num) <- colnames(GT)
    rownames(GT_num) <- var_ids

    idx0 <- which(GT == "0/0" | GT == "0|0")
    idx2 <- which(GT == "1/1" | GT == "1|1")
    idx1 <- which(GT == "0/1" | GT == "1/0" |
        GT == "0|1" | GT == "1|0")

    GT_num[idx0] <- 0
    GT_num[idx1] <- 1
    GT_num[idx2] <- 2

    if (na.rm) {
        idx <- rowSums(is.na(GT_num)) == 0
    } else {
        idx <- seq_len(nrow(GT_num))
    }
    GT_num <- GT_num[idx, ]

    ## Genotype probability
    if (keep_GP && sum(strsplit(GT_vcf@gt[1, 1], ":")$FORMAT == "GP") > 0) {
        GP_val <- matrix(NA, nrow = length(GT_num), ncol = 3)
        rownames(GP_val) <- paste0(
            rep(colnames(GT_num), each = ncol(GT_num)), ":",
            rep(rownames(GT_num), times = ncol(GT_num))
        )
        colnames(GP_val) <- c("GT=0", "GT=1", "GT=2")
        GP <- vcfR::extract.gt(GT_vcf, element = "GP", as.numeric = FALSE)
        for (ii in seq_len(3)) {
            GP_val[, ii] <- vcfR::masplit(GP,
                delim = ",", record = ii,
                sort = 0
            )[idx]
        }
    } else {
        GP_val <- NULL
    }

    list("GT" = GT_num, "GP" = GP_val)
}

### File: config_all

#' @name Config_all
#' @title A list of tree configuration
#' @description This list of tree configuration between 3 clones to 10 clones,
#' each element is a list with all possible tree matrix
#' @return NULL, but makes available a list
#' @docType data
#' @usage config_all
#' @format a list of list of matrix
#' @source PASTRI Python package
#' @author Yuanhua Huang, Davis McCarthy, 2018-06-25
NULL


### File: simulation_input

#' @name D_input
#' @title A matrix of sequencing depths
#' @description This matrix contains sequencing depths for 439 somatic variants
#' across 151 cells, from one particular scRNA-seq sample, can be used to
#' generate sequencing depths
#' @return NULL, but makes available a matrix
#' @docType data
#' @usage simulation_input
#' @format a matrix of float
#' @source A fibroblast sample from HipSci project
#' @author Yuanhua Huang, Davis McCarthy, 2018-06-25
NULL

#' @name tree_3clone
#' @title A tree object
#' @description This tree object with 3 clones contains clonal tree information,
#' inferred from bulk exome-seq data
#' @return NULL, but makes available a tree object
#' @docType data
#' @usage simulation_input
#' @format a tree object
#' @source A fibroblast sample from HipSci project
#' @author Yuanhua Huang, Davis McCarthy, 2018-06-25
NULL

#' @name tree_4clone
#' @title A tree object
#' @description This tree object with 4 clones contains clonal tree information,
#' inferred from bulk exome-seq data
#' @return NULL, but makes available a tree object
#' @docType data
#' @usage simulation_input
#' @format a tree object
#' @source A fibroblast sample from HipSci project
#' @author Yuanhua Huang, Davis McCarthy, 2018-06-25
NULL

#' @name tree_5clone
#' @title A tree object
#' @description This tree object with 5 clones contains clonal tree information,
#' inferred from bulk exome-seq data
#' @return NULL, but makes available a tree object
#' @docType data
#' @usage simulation_input
#' @format a tree object
#' @source A fibroblast sample from HipSci project
#' @author Yuanhua Huang, Davis McCarthy, 2018-06-25
NULL


### File: example_donor

#' @name tree
#' @title A tree object
#' @description This tree object contains clonal tree information, inferred
#' from bulk exome-seq data
#' @return NULL, but makes available a tree object
#' @docType data
#' @usage example_donor
#' @format a tree object
#' @source A fibroblast sample from HipSci project
#' @author Yuanhua Huang, Davis McCarthy, 2018-06-25
NULL

#' @name D_clone
#' @title A matrix of sequencing depths for clone ID
#' @description This matrix contains sequencing depths for 34 somatic variants
#' across 428 cells, from one example scRNA-seq sample
#' @return NULL, but makes available a matrix
#' @docType data
#' @usage example_donor
#' @format a matrix of float
#' @source A fibroblast sample from HipSci project
#' @author Yuanhua Huang, Davis McCarthy, 2018-06-25
NULL

#' @name A_clone
#' @title A matrix of read numbers of alternative alleles for clone ID
#' @description This matrix contains read numbers of alternative alleles for
#' 34 somatic variants across 428 cells, from one example scRNA-seq sample
#' @return NULL, but makes available a matrix
#' @docType data
#' @usage example_donor
#' @format a matrix of float
#' @source A fibroblast sample from HipSci project
#' @author Yuanhua Huang, Davis McCarthy, 2018-06-25
NULL

#' @name D_germline
#' @title A matrix of sequencing depths
#' @description This matrix contains sequencing depths for 34 germline variants
#' (near the somatic variants) across 428 cells, from one example scRNA-seq
#' sample
#' @return NULL, but makes available a matrix
#' @docType data
#' @usage example_donor
#' @format a matrix of float
#' @source A fibroblast sample from HipSci project
#' @author Yuanhua Huang, Davis McCarthy, 2018-06-25
NULL

#' @name A_germline
#' @title A matrix of read numbers of alternative alleles
#' @description This matrix contains read numbers of alternative alleles for
#' 34 germline variants (near the somatic variants) across 428 cells,
#' from one example scRNA-seq sample
#' @return NULL, but makes available a matrix
#' @docType data
#' @usage example_donor
#' @format a matrix of float
#' @source A fibroblast sample from HipSci project
#' @author Yuanhua Huang, Davis McCarthy, 2018-06-25
NULL

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_data.R
\name{read_vcf}
\alias{read_vcf}
\title{Read a VCF file into R session}
\usage{
read_vcf(
  vcf_file,
  genome = "GRCh37",
  seq_levels_style = "Ensembl",
  verbose = TRUE
)
}
\arguments{
\item{vcf_file}{character(1), path to VCF file to read into R session as a
\code{\link[VariantAnnotation]{CollapsedVCF}} object}

\item{genome}{character(1), string indicating the genome build used in the
VCF file(s) (default: "GRCh37")}

\item{seq_levels_style}{character(1), string passed to
\code{\link[GenomeInfoDb]{seqlevelsStyle}} the style to use for
chromosome/contig names (default: "Ensembl")}

\item{verbose}{logical(1), should messages be printed as function runs?}
}
\value{
a vcf object
}
\description{
Read a VCF file into R session
}
\examples{
vcf <- read_vcf(system.file("extdata", "cells.donorid.vcf.gz",
               package = "cardelino"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_data.R
\name{load_cellSNP_vcf}
\alias{load_cellSNP_vcf}
\title{Load sparse matrices A and D from cellSNP VCF file with filtering SNPs}
\usage{
load_cellSNP_vcf(
  vcf_file,
  min_count = 0,
  min_MAF = 0,
  max_other_allele = NULL,
  rowname_format = "full",
  keep_GL = FALSE
)
}
\arguments{
\item{vcf_file}{character(1), path to VCF file generated from cellSNP}

\item{min_count}{minimum count across all cells, e.g., 20}

\item{min_MAF}{minimum minor allele fraction, e.g., 0.1}

\item{max_other_allele}{maximum ratio of other alleles comparing to REF and
ALT alleles; for cellSNP vcf, we recommend 0.05}

\item{rowname_format}{the format of rowname: NULL is the default from vcfR,
short is CHROM_POS, and full is CHROM_POS_REF_ALT}

\item{keep_GL}{logical(1), if TRUE, check if GL (genotype probability) exists
it will be returned}
}
\value{
A list with elements the matrices A and D and GL, the genotype 
  probability. If keep_GL is false the GL element will be an empty list.
}
\description{
Load sparse matrices A and D from cellSNP VCF file with filtering SNPs
}
\examples{
vcf_file <- system.file("extdata", "cellSNP.cells.vcf.gz",
    package = "cardelino"
)
input_data <- load_cellSNP_vcf(vcf_file)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reads_simulator.R
\name{donor_read_simulator}
\alias{donor_read_simulator}
\title{Reads simulator for donor identification}
\usage{
donor_read_simulator(
  GT,
  D_seed,
  sample_variants = FALSE,
  donor_size = NULL,
  beta_shapes = NULL,
  n_cell = 5000,
  doublet_rate = NULL
)
}
\arguments{
\item{GT}{Variant-by-donor matrix for genotypes}

\item{D_seed}{Variant-by-cell matrix for read coverage for generating depth,
which be row sample and column sample both with replacement}

\item{sample_variants}{logical(1), if TRUE, sample variants with replacement
to the same size, otherwise not}

\item{donor_size}{Vector of float for the fractions of each donor; default
NULL means uniform}

\item{beta_shapes}{A 3-by-2 matrix of beta parameters for genotypes: 0, 1,
and 2; default NULL means matrix(c(0.2, 0.5, 99.8, 99.8, 0.5, 0.2), nrow = 3)}

\item{n_cell}{An integer for number of total cells}

\item{doublet_rate}{A float from 0 to 1 for doublet rate; default NULL means
rate n_cell / 100000}
}
\value{
A list of various components of the simulated dataset.
}
\description{
Reads simulator for donor identification
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clone_id.R
\name{Clone ID}
\alias{Clone ID}
\alias{clone_id}
\alias{clone_id_EM}
\alias{clone_id_Gibbs}
\title{Infer clonal identity of single cells}
\usage{
clone_id(
  A,
  D,
  Config = NULL,
  n_clone = NULL,
  Psi = NULL,
  relax_Config = TRUE,
  relax_rate_fixed = NULL,
  inference = "sampling",
  n_chain = 1,
  n_proc = 1,
  verbose = TRUE,
  ...
)

clone_id_EM(
  A,
  D,
  Config,
  Psi = NULL,
  min_iter = 10,
  max_iter = 1000,
  logLik_threshold = 1e-05,
  verbose = TRUE
)

clone_id_Gibbs(
  A,
  D,
  Config,
  Psi = NULL,
  relax_Config = TRUE,
  relax_rate_fixed = NULL,
  relax_rate_prior = c(1, 9),
  keep_base_clone = TRUE,
  prior0 = c(0.2, 99.8),
  prior1 = c(0.45, 0.55),
  min_iter = 5000,
  max_iter = 20000,
  buin_frac = 0.5,
  wise = "variant",
  relabel = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{A}{variant x cell matrix of integers; number of alternative allele
reads in variant i cell j}

\item{D}{variant x cell matrix of integers; number of total reads covering
variant i cell j}

\item{Config}{variant x clone matrix of binary values. The clone-variant
configuration, which encodes the phylogenetic tree structure. This is the
output Z of Canopy}

\item{n_clone}{integer(1), the number of clone to reconstruct. This is in use
only if Config is NULL}

\item{Psi}{A vector of float. The fractions of each clone, output P of Canopy}

\item{relax_Config}{logical(1), If TRUE, relaxing the Clone Configuration by
changing it from fixed value to act as a prior Config with a relax rate.}

\item{relax_rate_fixed}{numeric(1), If the value is between 0 to 1,
the relax rate will be set as a fix value during updating clone Config. If
NULL, the relax rate will be learned automatically with relax_rate_prior.}

\item{inference}{character(1), the method to use for inference, either
"sampling" to use Gibbs sampling (default) or "EM" to use
expectation-maximization (faster)}

\item{n_chain}{integer(1), the number of chains to run, which will be
averaged as an output result}

\item{n_proc}{integer(1), the number of processors to use. This parallel
computing can largely reduce time when using multiple chains}

\item{verbose}{logical(1), should the function output verbose information as
it runs?}

\item{...}{arguments passed to \code{\link{clone_id_Gibbs}} or
\code{\link{clone_id_EM}} (as appropriate)}

\item{min_iter}{A integer. The minimum number of iterations in the Gibbs
sampling. The real iteration may be longer until the convergence.}

\item{max_iter}{A integer. The maximum number of iterations in the Gibbs
sampling, even haven't passed the convergence diagnosis}

\item{logLik_threshold}{A float. The threshold of logLikelihood increase for
detecting convergence.}

\item{relax_rate_prior}{numeric(2), the two parameters of beta prior
distribution of the relax rate for relaxing the clone Configuration. This
mode is used when relax_relax is NULL.}

\item{keep_base_clone}{bool(1), if TRUE, keep the base clone of Config to its
input values when relax mode is used.}

\item{prior0}{numeric(2), alpha and beta parameters for the Beta prior
distribution on the inferred false positive rate.}

\item{prior1}{numeric(2), alpha and beta parameters for the Beta prior
distribution on the inferred (1 - false negative) rate.}

\item{buin_frac}{numeric(1), the fraction of chain as burn-in period}

\item{wise}{A string, the wise of parameters for theta1: global, variant,
element.}

\item{relabel}{bool(1), if TRUE, relabel the samples of both Config and prob
during the Gibbs sampling.}
}
\value{
If inference method is "EM", a list containing \code{theta}, a vector of
two floats denoting the parameters of the two components of the base model,
i.e., mean of Bernoulli or binomial model given variant exists or not,
\code{prob}, the matrix of posterior probabilities of each cell belonging to
each clone with fitted parameters, and \code{logLik}, the log likelihood of
the final parameters.

If inference method is "sampling", a list containing: \code{theta0}, the mean
of sampled false positive parameter values; \code{theta1} the mean of sampled
(1 - false negative rate) parameter values; \code{theta0_all}, all sampled
false positive parameter values; \code{theta1_all}, all sampled (1 - false
negative rate) parameter values; \code{element}; \code{logLik_all},
log-likelihood for model for all sampled parameter sets; \code{prob_all};
\code{prob}, matrix with mean of sampled cell-clone assignment posterior
probabilities (the key output of the model); \code{prob_variant}.

a list containing \code{theta}, a vector of two floats denoting the
binomial rates given variant exists or not, \code{prob}, the matrix of
posterior probabilities of each cell belonging to each clone with fitted
parameters, and \code{logLik}, the log likelihood of the final parameters.
}
\description{
Infer clonal identity of single cells

Assign cells to clones using an EM algorithm

Assign cells to clones using a Gibbs sampling algorithm
}
\details{
The two Bernoulli components correspond to false positive and false negative
rates. The two binomial components correspond to the read distributions
with and without the mutation present.
}
\examples{
data(example_donor)
assignments <- clone_id(A_clone, D_clone,
    Config = tree$Z,
    min_iter = 800, max_iter = 1200
)
prob_heatmap(assignments$prob)

assignments_EM <- clone_id(A_clone, D_clone,
    Config = tree$Z,
    inference = "EM"
)
prob_heatmap(assignments_EM$prob)
}
\author{
Yuanhua Huang and Davis McCarthy

Yuanhua Huang
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binomial_models.R
\name{mixBinom}
\alias{mixBinom}
\title{EM algorithm for estimating binomial mixture model}
\usage{
mixBinom(
  k,
  n,
  n_components = 2,
  p_init = NULL,
  learn_p = TRUE,
  min_iter = 10,
  max_iter = 1000,
  logLik_threshold = 1e-05
)
}
\arguments{
\item{k}{A vector of integers. number of success}

\item{n}{A vector of integers. number of trials}

\item{n_components}{A number. number of components}

\item{p_init}{A vector of floats with length n_components, the initial value
of p}

\item{learn_p}{bool(1) or a vector of bool, whether learn each p}

\item{min_iter}{integer(1). number of minimum iterations}

\item{max_iter}{integer(1). number of maximum iterations}

\item{logLik_threshold}{A float. The threshold of logLikelihood increase for
detecting convergence}
}
\value{
a list containing \code{p}, a vector of floats between 0 and 1
giving the estimated success probability for each component, \code{psi},
estimated fraction of each component in the mixture, and \code{prob}, the
matrix of fitted probabilities of each observation belonging to each
component.
}
\description{
EM algorithm for estimating binomial mixture model
}
\examples{
n1 <- array(sample(1:30, 50, replace = TRUE))
n2 <- array(sample(1:30, 200, replace = TRUE))
k1 <- apply(n1, 1, rbinom, n = 1, p = 0.5)
k2 <- apply(n2, 1, rbinom, n = 1, p = 0.01)
RV <- mixBinom(c(k1, k2), c(n1, n2))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_config_diffs}
\alias{plot_config_diffs}
\title{Define a publication-style plot theme}
\usage{
plot_config_diffs(Config1, Config2, show_variant_names = FALSE)
}
\arguments{
\item{Config1}{variant by clone matrix defining the first clonal structure}

\item{Config2}{variant by clone matrix defining the second clonal structure}

\item{show_variant_names}{logical(1), should the variant names (rownames of
Config matrices) be shown on the plot? Default is \code{FALSE}.}
}
\value{
a ggplot heatmap style plot showing the differences between the two
Config matrices, specifically the differences Config1 - Config2.
}
\description{
Define a publication-style plot theme
}
\examples{
Config1 <- matrix(c(
    rep(0, 15), rep(1, 8), rep(0, 7),
    rep(1, 5), rep(0, 3), rep(1, 7)
), ncol = 3)
Config2 <- matrix(c(
    rep(0, 15), rep(1, 8), rep(1, 7),
    rep(0, 5), rep(1, 3), rep(1, 7)
), ncol = 3)
rownames(Config1) <- rownames(Config2) <- paste0("var", 1:nrow(Config1))
colnames(Config1) <- colnames(Config2) <- paste0("clone", 1:ncol(Config1))
plot_config_diffs(Config1, Config2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clone_id.R
\name{devianceIC}
\alias{devianceIC}
\title{Deviance Information Criterion for cardelino model}
\usage{
devianceIC(logLik_all, logLik_post)
}
\arguments{
\item{logLik_all}{A vector of numeric; the log likelihood of posterior
sample, i.e., posterior samples of deviance}

\item{logLik_post}{numeric(1); the log likelihood of mean posterior
parameters, i.e., deviance of posterior means}
}
\value{
\code{DIC}, a float of deviance information criterion
}
\description{
Deviance Information Criterion for cardelino model
}
\author{
Yuanhua Huang
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_tree}
\alias{plot_tree}
\title{Plot a phylogenetic tree}
\usage{
plot_tree(tree, orient = "h")
}
\arguments{
\item{tree}{A phylgenetic tee object of class "phylo"}

\item{orient}{A string for the orientation of the tree: "v" (vertical) or "h"
(horizontal)}
}
\value{
a ggtree object
}
\description{
Plot a phylogenetic tree
}
\details{
This function plots a phylogenetic tree from an object of class
"phylo", as produced, for example, by the Canopy package.
}
\examples{
data(example_donor)
plot_tree(tree, orient = "v")
}
\references{
This function makes use of the \code{\link{ggtree}} package:

Guangchuang Yu, David Smith, Huachen Zhu, Yi Guan, Tommy Tsan-Yuk Lam.
ggtree: an R package for visualization and annotation of phylogenetic trees
with their covariates and other associated data. Methods in Ecology and
Evolution 2017, 8(1):28-36, doi:10.1111/2041-210X.12628
}
\author{
Davis McCarthy and Yuanhua Huang
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{pub.theme}
\alias{pub.theme}
\title{Define a publication-style plot theme}
\usage{
pub.theme(size = 12)
}
\arguments{
\item{size}{numeric, base font size for adapted ggplot2 theme}
}
\value{
a ggplot theme based on theme_classic
}
\description{
Define a publication-style plot theme
}
\details{
This theme modifies the \code{\link[ggplot2]{theme_classic}} theme
in ggplot2.
}
\examples{
library(ggplot2)
x <- sample(10)
y <- x + runif(10) - 0.5
df <- data.frame(x = x, y = y)
fig <- ggplot(df, aes(x = x, y = y)) +
    geom_point() +
    pub.theme()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tree_utils.R
\name{get_tree}
\alias{get_tree}
\title{Get a clonal tree from a configuration matrix}
\usage{
get_tree(Config, P = NULL, strictness = "lax")
}
\arguments{
\item{Config}{variant x clone matrix of binary values. The clone-variant
configuration, which encodes the phylogenetic tree structure. This is the
output Z of Canopy}

\item{P}{a one-column numeric matrix encoding the (observed or estimated)
prevalence (or frequency) of each clone}

\item{strictness}{character(1), a character string defining the strictness of
the function if there are all-zero rows in the Config matrix. If \code{"lax"}
then the function silently drops all-zero rows and proceeds. If \code{"warn"}
then the function warns of dropping all-zero rows and proceeds. If
\code{"error"} then the function throws an error is all-zero rows are
detected.}
}
\value{
An object of class "phylo" describing the tree structure. The output object
also contains an element "sna" defining the clustering of variants onto the
branches of the tree, and if \code{P} is non-null it also contains VAF
(variant allele frequency), CCF (cell clone fraction) and clone prevalence
values (computed from the supplied \code{P} argument).
}
\description{
Get a clonal tree from a configuration matrix
}
\details{
Output tree may be nonsensical if the input \code{Config} matrix does not
define a coherent tree structure.
}
\examples{
Configk3 <- matrix(c(
    rep(0, 15), rep(1, 8), rep(0, 7), rep(1, 5), rep(0, 3),
    rep(1, 7)
), ncol = 3)
tree_k3 <- get_tree(Config = Configk3, P = matrix(rep(1 / 3, 3), ncol = 1))
plot_tree(tree_k3)
}
\author{
Davis McCarthy
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reads_simulator.R
\name{sim_read_count}
\alias{sim_read_count}
\title{Synthetic reads generator for genetic variants}
\usage{
sim_read_count(
  Config,
  D,
  Psi = NULL,
  means = c(0.002, 0.45),
  vars = c(100, 1),
  wise0 = "element",
  wise1 = "variant",
  cell_num = 300,
  permute_D = FALSE,
  sample_cell = TRUE,
  doublet = 0
)
}
\arguments{
\item{Config}{A matrix of binary values. The clone-variant configuration,
which encodes the phylogenetic tree structure, and the genotype of each clone}

\item{D}{A matrix of integers. Sequencing depth for N variants across x
cells (ideally >100 cells). NA means 0 here.}

\item{Psi}{A vector of float. The fractions of each clone. If NULL, set a
uniform distribution.}

\item{means}{A vector of two floats. The mean theta_1 (false positive rate)
and the mean theta_2 (true positive rate).}

\item{vars}{A vector of two floats. The variance of theta_1 and theta_2.}

\item{wise0}{A string, the beta-binomial parameter specificity for theta0:
global, variant, element.}

\item{wise1}{A string, the beta-binomial parameter specificity for theta1:
global, variant, element.}

\item{cell_num}{A integer. The number of cells to generate.}

\item{permute_D}{A Boolean value. If True permute variants in D.}

\item{sample_cell}{A Boolean value. If True and M > ncol(D), sample cells.}

\item{doublet}{A float between 0 and 1, the rate of doublets}
}
\value{
a list containing \code{A_sim}, a matrix for alteration reads,
\code{A_sim}, a matrix for total reads, \code{I_sim}, a matrix for clonal
label, \code{H_sim}, a matrix for genotype, \code{theta0}, a matrix of
expected false positive rate, \code{theta1}, a matrix of expected true
positive rate, \code{theta0_binom}, theta0 as binomial parameter,
\code{theta1_binom}, theta0 as binomial parameter, and \code{is_doublet}, a
vector of Boolean value if a cell is a doublet
}
\description{
There are following steps to generate the simulated reads counts for variants
in single cells:
1) given the clonal genotype and the clonal prevalence, the genotypes (i.e,
the clone) of cells will be generated following a multinomial distribution.
Note, one cell may contain variants from two clones when it is a doublet.
2) given the distribution of reads coverage, e.g., a matrix of read coverage
from real data, (variant specific), the total reads of each variant will be
generated by random sampling. Note, the missing rate is governed by this
matrix.
3) the allelic frequency of each variant will be generated by following a
beta distribution with parameters of mean and variance.
4) Given the genotype of a cell, if the mutation exists in a cell, the
alteration read counts will be generated by a binomial distribution,
parameterized the allelic frequency, sampled from step 3.
5) Given the genotype of a cell, if the mutation does not exist in a cell,
the alteration read counts will be generated by a binomial distribution,
parameterized by the technical error rate.
}
\examples{
data(simulation_input)
D2 <- sample_seq_depth(D_input, n_cells = 500, n_sites = nrow(tree_4clone$Z))
simu <- sim_read_count(tree_4clone$Z, D2, Psi = NULL, cell_num = 500)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assessment.R
\name{mtx_to_df}
\alias{mtx_to_df}
\title{Convert a matrix to data frame}
\usage{
mtx_to_df(X)
}
\arguments{
\item{X}{A matrix of values}
}
\value{
A data.frame version of the passed matrix.
}
\description{
Convert a matrix to data frame
}
\examples{
mtx_to_df(matrix(seq(12), nrow = 3))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_data.R
\name{get_snp_matrices}
\alias{get_snp_matrices}
\title{Get SNP data matrices from VCF object(s)}
\usage{
get_snp_matrices(vcf_cell, vcf_donor = NULL, verbose = TRUE, donors = NULL)
}
\arguments{
\item{vcf_cell}{a \code{\link[VariantAnnotation]{CollapsedVCF}} object
containing variant data for cells}

\item{vcf_donor}{an optional \code{\link[VariantAnnotation]{CollapsedVCF}}
object containing genotype data for donors}

\item{verbose}{logical(1), should the function output verbose information as
it runs?}

\item{donors}{optional character vector providing a set of donors to use, by
subsetting the donors present in the \code{donor_vcf_file}; if \code{NULL}
(default) then all donors present in VCF will be used.}
}
\value{
a list containing
\code{A}, a matrix of integers. Number of alteration reads in SNP i cell j.
\code{D}, a matrix of integers. Number of reads depth in SNP i cell j.
\code{R}, a matrix of integers. Number of reference reads in SNP i cell j.
\code{GT_cells}, a matrix of integers for genotypes. The cell-SNP
configuration.
\code{GT_donors}, a matrix of integers for genotypes. The donor-SNP
configuration.
}
\description{
Get SNP data matrices from VCF object(s)
}
\examples{
vcf_cell <- read_vcf(system.file("extdata", "cells.donorid.vcf.gz",
                     package = "cardelino"))
vcf_donor <-  read_vcf(system.file("extdata", "donors.donorid.vcf.gz",
                       package = "cardelino"))
snp_data <- get_snp_matrices(vcf_cell, vcf_donor)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_data.R
\name{load_GT_vcf}
\alias{load_GT_vcf}
\title{Load genotype VCF into numeric values: 0, 1, or 2}
\usage{
load_GT_vcf(vcf_file, rowname_format = "full", na.rm = TRUE, keep_GP = TRUE)
}
\arguments{
\item{vcf_file}{character(1), path to VCF file for donor genotypes}

\item{rowname_format}{the format of rowname: NULL is the default from vcfR,
short is CHROM_POS, and full is CHROM_POS_REF_ALT}

\item{na.rm}{logical(1), if TRUE, remove the variants with NA values}

\item{keep_GP}{logical(1), if TRUE, check if GP (genotype probability) exists
it will be returned}
}
\value{
A list representing the loaded genotype information with two
  components: GT, the usual numeric representation of genotype and GP the
  genotype probabilities. Note that if \code{keep_GP} is false the GP
  component will be NULL.
}
\description{
Note, the genotype VCF can be very big for whole genome. It would be more
efficient to only keep the wanted variants and samples. bcftools does such
jobs nicely.
}
\examples{
vcf_file <- system.file("extdata", "cellSNP.cells.vcf.gz",
    package = "cardelino"
)
GT_dat <- load_GT_vcf(vcf_file, na.rm = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reads_simulator.R
\name{sample_tree_SNV}
\alias{sample_tree_SNV}
\title{Down sample number of SNVs in the tree}
\usage{
sample_tree_SNV(tree, n_SNV = NULL)
}
\arguments{
\item{tree}{A tree object from Canopy}

\item{n_SNV}{A integer, the number of SNVs to keep in the output tree}
}
\value{
a phylo tree with down sampled variants
}
\description{
Down sample number of SNVs in the tree
}
\examples{
data(simulation_input)
tree_lite <- sample_tree_SNV(tree_4clone, n_SNV = 10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binomial_models.R
\name{predMixBinom}
\alias{predMixBinom}
\title{Predicted probability from learned binomial mixture model}
\usage{
predMixBinom(k, n, p, psi)
}
\arguments{
\item{k}{A vector of integers. number of success}

\item{n}{A vector of integers. number of trials}

\item{p}{a vector of binomial success probabilities}

\item{psi}{A float between 0 and 1. fraction of each component}
}
\value{
A list with two components: prob, a matrix representing the 
  probability of each of the passed values coming from each component of the
  mixture and logLik, the total log-likelihood of the new samples.
}
\description{
Predicted probability from learned binomial mixture model
}
\examples{
n1 <- array(sample(1:30, 50, replace = TRUE))
n2 <- array(sample(1:30, 200, replace = TRUE))
k1 <- apply(n1, 1, rbinom, n = 1, p = 0.5)
k2 <- apply(n2, 1, rbinom, n = 1, p = 0.01)
RV <- mixBinom(c(k1, k2), c(n1, n2))
RV_pred <- predMixBinom(3, 10, RV$p, RV$psi)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_info.R
\docType{data}
\name{A_clone}
\alias{A_clone}
\title{A matrix of read numbers of alternative alleles for clone ID}
\format{
a matrix of float
}
\source{
A fibroblast sample from HipSci project
}
\usage{
example_donor
}
\value{
NULL, but makes available a matrix
}
\description{
This matrix contains read numbers of alternative alleles for
34 somatic variants across 428 cells, from one example scRNA-seq sample
}
\author{
Yuanhua Huang, Davis McCarthy, 2018-06-25
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{heat_matrix}
\alias{heat_matrix}
\title{Plot heatmap from a matrix}
\usage{
heat_matrix(mat, base_size = 12, digits = 2, show_value = FALSE)
}
\arguments{
\item{mat}{A matrix to show, column by x-axis and row by y-axis}

\item{base_size}{Numeric value for the base size in theme_bw}

\item{digits}{Integer value for the number of digits to show}

\item{show_value}{Logical value for showing the value for each element or not}
}
\value{
A ggplot heatmap visualization of the passed matrix.
}
\description{
Plot heatmap from a matrix
}
\examples{
mat <- matrix(rnorm(9), ncol = 3, nrow = 3) + diag(rnorm(3, 2, 0.1))
rownames(mat) <- paste0("sample_", letters[1:3])
colnames(mat) <- paste0("var_", 1:3)
heat_matrix(mat)

# Additional arguments. 
heat_matrix(mat, base_size = 6)
heat_matrix(mat, show_value = TRUE)
heat_matrix(mat, show_value = TRUE, digits = 4)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assessment.R
\name{binaryPRC}
\alias{binaryPRC}
\title{Precision-recall curve for binary label prediction}
\usage{
binaryPRC(
  scores,
  labels,
  cutoff = NULL,
  cut_direction = ">=",
  add_cut1 = FALSE,
  empty_precision = 1
)
}
\arguments{
\item{scores}{Prediction score for each sample}

\item{labels}{True labels for each sample, e.g., from simulation}

\item{cutoff}{A vector of cutoffs; if NULL use all unique scores}

\item{cut_direction}{A string to compare with cutoff: >=, >, <=, <}

\item{add_cut1}{Logical value; if True, manually add a cutoff of 1}

\item{empty_precision}{Float value for default precision if no any recall}
}
\value{
A data.frame containing recall and precision values at various 
cutoffs.
}
\description{
Precision-recall curve for binary label prediction
}
\examples{
scores <- 1:10
labels <- c(0, 0, 0, 1, 0, 1, 0, 1, 1, 1)
binaryPRC(scores, labels)

# Extra arguments. 
binaryPRC(scores, labels, cutoff = seq(1, 10, by = 2))
binaryPRC(scores, labels, cut_direction = ">")
binaryPRC(scores, labels, add_cut1 = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{vc_heatmap}
\alias{vc_heatmap}
\title{Plot a variant-cell heatmap for cell clonal assignment}
\usage{
vc_heatmap(mat, prob, Config, show_legend = FALSE)
}
\arguments{
\item{mat}{A matrix for heatmap: N variants x M cells. row and column will be
sorted automatically.}

\item{prob}{A matrix of probability of clonal assignment: M cells x K clones}

\item{Config}{A binary matrix of clonal Configuration: N variants x K clones}

\item{show_legend}{A bool value: if TRUE, show the legend}
}
\value{
a pheatmap object

a ggplot object
}
\description{
Plot a variant-cell heatmap for cell clonal assignment
}
\examples{
data(example_donor)
assignments <- clone_id(A_clone, D_clone, Config = tree$Z)
fig <- vc_heatmap(assignments$prob_variant, assignments$prob, tree$Z)
}
\references{
This function makes use of the \code{\link{pheatmap}} packages
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{heatmap.theme}
\alias{heatmap.theme}
\title{The theme of heatmaps for prob_heatmap and sites_heatmap}
\usage{
heatmap.theme(legend.position = "bottom", size = 12)
}
\arguments{
\item{legend.position}{character, describes where to place legend on plot
(passed to \code{\link[ggplot2]{theme_gray}})}

\item{size}{numeric, base font size for plot (passed to
\code{\link[ggplot2]{theme_gray}})}
}
\value{
a ggplot theme based on theme_gray
}
\description{
The theme of heatmaps for prob_heatmap and sites_heatmap
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clone_id.R
\name{Geweke_Z}
\alias{Geweke_Z}
\title{Geweke diagnostic for MCMC sampling.}
\usage{
Geweke_Z(X, first = 0.1, last = 0.5)
}
\arguments{
\item{X}{A matrix of MCMC samples for N samples per K variables}

\item{first}{A float between 0 and 1. The initial region of MCMC chain.}

\item{last}{A float between 0 and 1. The final region of MCMC chain.}
}
\value{
\code{Z}, a vector of absolute value of Z scores for each variable.
When |Z| <= 2, the sampling could be taken as converged.
}
\description{
Geweke diagnostic for MCMC sampling.
}
\author{
Yuanhua Huang
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assessment.R
\name{rowArgmax}
\alias{rowArgmax}
\title{Column index of the maximum value for each row in a matrix}
\usage{
rowArgmax(X)
}
\arguments{
\item{X}{A matrix of floats.}
}
\value{
a vector of the index of column for each row. Note, when multiple
columns have the same value, only the earliest column will be
returned.
}
\description{
Column index of the maximum value for each row in a matrix
}
\examples{
matA <- matrix(sample(seq(12)), nrow = 3)
rowArgmax(matA)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assessment.R
\name{assign_scores}
\alias{assign_scores}
\title{Scoring the simulation in assignment of singlets and doublets}
\usage{
assign_scores(prob, I_sim, cutoff = seq(0, 1, 0.001))
}
\arguments{
\item{prob}{Probability matrix for each cell to each component}

\item{I_sim}{The true identity of assignment from simulation}

\item{cutoff}{A list of cutoffs from 0 to 1}
}
\value{
A list with components: df_sg, the recall/precision data.frame 
calculated by \code{multiPRC()}, AUC_sg, the AUC calculated by 
\code{multiPRC()}, df_db, the recall/precision data.frame calculated by
\code{binaryPRC()} and AUC_db the AUC calculated by \code{binaryPRC()}. 
Note that \code{multiPRC()} is run on a multiclass version of the problem
and \code{binaryPRC} is run on a binarised version of the problem.
}
\description{
Scoring the simulation in assignment of singlets and doublets
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assessment.R
\name{rowMax}
\alias{rowMax}
\title{Maximum value for each row in a matrix}
\usage{
rowMax(X, mode = "best")
}
\arguments{
\item{X}{A matrix of floats.}

\item{mode}{A string, the method for defining scores for filtering cells:
best, second and delta. \code{best}: highest value for each row, similarly
for the \code{second}. \code{delta} is the difference between the best and
the second.}
}
\value{
a vector of the collapsed value for each row, depending
on the mode used.
}
\description{
Maximum value for each row in a matrix
}
\examples{
matA <- matrix(sample(seq(12)), nrow = 3)
rowMax(matA)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_info.R
\docType{data}
\name{Config_all}
\alias{Config_all}
\title{A list of tree configuration}
\format{
a list of list of matrix
}
\source{
PASTRI Python package
}
\usage{
config_all
}
\value{
NULL, but makes available a list
}
\description{
This list of tree configuration between 3 clones to 10 clones,
each element is a list with all possible tree matrix
}
\author{
Yuanhua Huang, Davis McCarthy, 2018-06-25
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assessment.R
\name{colMatch}
\alias{colMatch}
\title{Column match between two matrices by minimum mean absolute difference}
\usage{
colMatch(A, B, force = FALSE)
}
\arguments{
\item{A}{The first matrix which will be matched}

\item{B}{The second matrix, the return index will be used on}

\item{force}{bool(1), If TRUE, force traversing all permutations of B to
find the optimised match to A with computing cost of O(n!). Otherwise, use
greedy search with computing cost of O(n^2).}
}
\value{
\code{idx}, the column index of B to be matched to A
}
\description{
Column match between two matrices by minimum mean absolute difference
}
\examples{
matA <- matrix(sample(seq(12)), nrow = 3)
col_idx <- sample(4)
matB <- matA[, col_idx]
colMatch(matB, matA)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clone_id.R
\name{assign_cells_to_clones}
\alias{assign_cells_to_clones}
\title{Assign cells to clones from cardelino results}
\usage{
assign_cells_to_clones(prob_mat, threshold = 0.5)
}
\arguments{
\item{prob_mat}{numeric matrix (cells x clones) of clone posterior
probabilities as output by \code{\link{clone_id}}}

\item{threshold}{numeric(1), posterior probability threshold for cell-clone
assignment: if posterior probability is above threshold, assign cell to
clone, otherwise leave cell "unassigned"}
}
\value{
a \code{data.frame} with cell ID, assigned clone label and maximum
posterior probability across clones.
}
\description{
Assign cells to clones from cardelino results
}
\examples{
data(example_donor)
assignments <- clone_id(A_clone, D_clone, Config = tree$Z, inference = "EM")
df <- assign_cells_to_clones(assignments$prob)
head(df)
table(df$clone)
}
\author{
Davis McCarthy
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reads_simulator.R
\name{sample_seq_depth}
\alias{sample_seq_depth}
\title{Update matrix D with manually selected missing rate}
\usage{
sample_seq_depth(D, n_cells = NULL, n_sites = NULL, missing_rate = NULL)
}
\arguments{
\item{D}{A matrix (N variants x M cells), the original sequencing coverage,
NA means missing}

\item{n_cells}{A integer, the number of the cells to generate}

\item{n_sites}{A integer, the number of variants to generate}

\item{missing_rate}{A float value, if NULL, use the same missing rate as D}
}
\value{
a n_sites by n_cells matrix sampled from input D.
}
\description{
Given missing rate, the NA will be generated first. For none NA element,
sequencing depth with uniformly sampled from D, row wisely. Namely, the depth
is variant specific.
}
\examples{
data(simulation_input)
D1 <- sample_seq_depth(D_input,
    n_cells = 500, n_sites = 50,
    missing_rate = 0.85
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_info.R
\docType{data}
\name{tree_4clone}
\alias{tree_4clone}
\title{A tree object}
\format{
a tree object
}
\source{
A fibroblast sample from HipSci project
}
\usage{
simulation_input
}
\value{
NULL, but makes available a tree object
}
\description{
This tree object with 4 clones contains clonal tree information,
inferred from bulk exome-seq data
}
\author{
Yuanhua Huang, Davis McCarthy, 2018-06-25
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_info.R
\docType{data}
\name{D_germline}
\alias{D_germline}
\title{A matrix of sequencing depths}
\format{
a matrix of float
}
\source{
A fibroblast sample from HipSci project
}
\usage{
example_donor
}
\value{
NULL, but makes available a matrix
}
\description{
This matrix contains sequencing depths for 34 germline variants
(near the somatic variants) across 428 cells, from one example scRNA-seq
sample
}
\author{
Yuanhua Huang, Davis McCarthy, 2018-06-25
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clone_id.R
\name{get_logLik}
\alias{get_logLik}
\title{Log likelihood of clone_id model
It returns P(A, D | C, I, theta0, theta1)}
\usage{
get_logLik(A1, B1, Config, Assign, theta0, theta1)
}
\arguments{
\item{A1}{variant x cell matrix of integers; number of alternative allele
reads in variant i cell j}

\item{B1}{variant x cell matrix of integers; number of reference allele
reads in variant i cell j}

\item{Config}{variant x clone matrix of float values. The clone-variant
configuration probability, averaged by posterior samples}

\item{Assign}{cells x clone matrix of float values. The cell-clone
assignment probability, averaged by posterior samples}

\item{theta0}{the binomial rate for alternative allele from config = 0}

\item{theta1}{the binomial rate for alternative allele from config = 1}
}
\value{
\code{logLik}, a float of log likelihood
}
\description{
Log likelihood of clone_id model
It returns P(A, D | C, I, theta0, theta1)
}
\author{
Yuanhua Huang
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{prob_heatmap}
\alias{prob_heatmap}
\title{Plot a heatmap for probability of clone assignment}
\usage{
prob_heatmap(prob_mat, threshold = 0.5, mode = "best", cell_idx = NULL)
}
\arguments{
\item{prob_mat}{A matrix (M x K), the probability of cell j to clone k}

\item{threshold}{A float value, the threshold for assignable cells}

\item{mode}{A string, the method for defining scores for filtering cells:
best and delta. best: highest probability of a cell to K clones, delta: the
difference between the best and second.}

\item{cell_idx}{A vector the indices of the input cells. If NULL, order by
the probability of each clone}
}
\value{
a ggplot object
}
\description{
Plot a heatmap for probability of clone assignment
}
\examples{
data(example_donor)
assignments <- clone_id(A_clone, D_clone, Config = tree$Z, inference = "EM")
fig <- prob_heatmap(assignments$prob)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_info.R
\docType{data}
\name{D_input}
\alias{D_input}
\title{A matrix of sequencing depths}
\format{
a matrix of float
}
\source{
A fibroblast sample from HipSci project
}
\usage{
simulation_input
}
\value{
NULL, but makes available a matrix
}
\description{
This matrix contains sequencing depths for 439 somatic variants
across 151 cells, from one particular scRNA-seq sample, can be used to
generate sequencing depths
}
\author{
Yuanhua Huang, Davis McCarthy, 2018-06-25
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assessment.R
\name{multiPRC}
\alias{multiPRC}
\title{Precision-recall curve for multi-class prediction}
\usage{
multiPRC(
  prob_mat,
  simu_mat,
  marginal_mode = "best",
  cutoff = NULL,
  multiLabel.rm = TRUE,
  add_cut1 = FALSE
)
}
\arguments{
\item{prob_mat}{Probability matrix for each cell to each component}

\item{simu_mat}{The true identity of assignment from simulation}

\item{marginal_mode}{A string for the mode to marginalize the column: best,
second, or delta}

\item{cutoff}{A list of cutoff; if NULL use all unique scores}

\item{multiLabel.rm}{Logical value; if True, remove the samples with
multiple labels}

\item{add_cut1}{Logical value; if True, manually add a cutoff of 1}
}
\value{
A list with two components: df, a data.frame containing precision 
and recall values at various cutoffs and AUC, the overall AUC.
}
\description{
Precision-recall curve for multi-class prediction
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assessment.R
\name{binaryROC}
\alias{binaryROC}
\title{ROC curve for binary label prediction}
\usage{
binaryROC(
  scores,
  labels,
  cutoff = NULL,
  cut_direction = ">=",
  add_cut1 = TRUE,
  cutoff_point = 0.9
)
}
\arguments{
\item{scores}{Prediction score for each sample}

\item{labels}{True labels for each sample, e.g., from simulation}

\item{cutoff}{A vector of cutoffs; if NULL use all unique scores}

\item{cut_direction}{A string to compare with cutoff: >=, >, <=, <}

\item{add_cut1}{Logical value; if True, manually add a cutoff of 1}

\item{cutoff_point}{Numeric value; additional cutoff value}
}
\value{
A data.frame containing AUC and AUPRC at various cutoffs.
}
\description{
ROC curve for binary label prediction
}
\examples{
scores <- 1:10
labels <- c(0, 0, 0, 1, 0, 1, 0, 1, 1, 1)
binaryROC(scores, labels)

# Extra arguments. 
binaryROC(scores, labels, cutoff = seq(1, 10, by = 2))
binaryROC(scores, labels, cut_direction = ">")
binaryROC(scores, labels, add_cut1 = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_info.R
\docType{data}
\name{tree}
\alias{tree}
\title{A tree object}
\format{
a tree object
}
\source{
A fibroblast sample from HipSci project
}
\usage{
example_donor
}
\value{
NULL, but makes available a tree object
}
\description{
This tree object contains clonal tree information, inferred
from bulk exome-seq data
}
\author{
Yuanhua Huang, Davis McCarthy, 2018-06-25
}
